import logging

import pytest

from ansible_galaxy import repository_spec_parse
from ansible_galaxy import exceptions

log = logging.getLogger(__name__)


split_kwarg_valid_test_cases = \
    ['something',
     '1.2.3',
     'version=1.2.3',
     'name=somename',

     # weird but valid
     'name==',
     'name=name',
     'name=version',
     'name=war=peace',

     ]


@pytest.fixture(scope='module',
                params=split_kwarg_valid_test_cases)
def split_kwarg_valid(request):
    yield request.param


def test_split_kwarg_valid(split_kwarg_valid):
    valid_keywords = ('name', 'version')
    result = repository_spec_parse.split_kwarg(split_kwarg_valid, valid_keywords)
    log.debug('spec=%s result=%s', split_kwarg_valid, [x for x in result])


split_kwarg_invalid_test_cases = \
    ['=',
     'some_invalid_keyword=foo',
     'blip=',
     ]


@pytest.fixture(scope='module',
                params=split_kwarg_invalid_test_cases)
def split_kwarg_invalid(request):
    yield request.param


def test_split_kwarg_invalid(split_kwarg_invalid):
    valid_keywords = ('name', 'version')
    try:
        repository_spec_parse.split_kwarg(split_kwarg_invalid, valid_keywords)
    except exceptions.GalaxyClientError as e:
        log.exception(e)
        return

    assert False, 'Expected to get a GalaxyClientError for invalid kwargs for %s' % split_kwarg_invalid


split_comma_test_cases = \
    ['foo',
     'foo,1.2.3',
     'foo,version=1.2.3',
     'foo,1.2.3,somename',
     'foo,1.2.3,name=somename',
     'foo,1.2.3,somename,somescm',
     'foo,1.2.3,somename,somescm,someextra'
     ]


@pytest.fixture(scope='module',
                params=split_comma_test_cases)
def split_comma(request):
    yield request.param


def test_split_comma(split_comma):
    valid_keywords = ('name', 'version')
    result = repository_spec_parse.split_comma(split_comma, valid_keywords)
    log.debug('spec=%s result=%s', split_comma, [x for x in result])


just_src = {'src': 'something'}
src_ver = {'src': 'something',
           'version': '1.2.3'}
src_name = {'src': 'something',
            'name': 'somename'}
full_info = {'src': 'something',
             'version': '1.2.3',
             'name': 'somename'}
split_repository_spec_test_cases = \
    [('something', just_src),
     ('something,1.2.3', src_ver),
     ('something,version=1.2.3', src_ver),
     ('something,1.2.3,somename', full_info),
     ('something,1.2.3,name=somename', full_info),
     ('something,name=somename,version=1.2.3', full_info),
     ('something,1.2.3,somename', full_info),
     # dont want to expect this to work
     # ('something,name=somename,1.2.3', full_info),
     ]


@pytest.fixture(scope='module',
                params=split_repository_spec_test_cases,
                ids=[x[0] for x in split_repository_spec_test_cases])
def split_repository_spec_fixture(request):
    yield request.param


def test_split_repository_spec(split_repository_spec_fixture):
    valid_keywords = ('src', 'version', 'name', 'scm')
    result = repository_spec_parse.split_repository_spec(split_repository_spec_fixture[0], valid_keywords)
    log.debug('spec=%s result=%s', split_repository_spec_fixture[0], result)
    assert result == split_repository_spec_fixture[1]


def assert_keys(repository_spec, name=None, version=None,
                scm=None, src=None, namespace=None):
    # name = name or ''
    # src = src or ''
    assert isinstance(repository_spec, dict)

    log.debug('repository_spec: %s', repository_spec)
    # TODO: should it default to empty string?
    assert repository_spec['name'] == name, \
        'repository_spec name=%s does not match expected name=%s' % (repository_spec['name'], name)
    assert repository_spec['version'] == version
    assert repository_spec['scm'] == scm
    assert repository_spec['src'] == src, \
        'repository_spec src=%s does not match expected src=%s' % (repository_spec['src'], src)


def parse_repository_spec(repository_spec_string, resolver=None):
    result = repository_spec_parse.spec_data_from_string(repository_spec_string,
                                                         resolver=resolver)
    log.debug('result: %s', result)
    return result


def assert_just_keys(parse_result):
    valid_keys = ('name', 'namespace', 'src', 'scm', 'version', 'spec_string', 'fetch_method')

    for key in valid_keys:
        assert key in parse_result, 'expected the results dict to have a "%s" key but it did not' % key

    for result_key in parse_result.keys():
        assert result_key in valid_keys, 'the results had unexpected key="%s"' % result_key


def test_parse_repository_spec_src_no_namespace_required():
    spec_text = 'some_content'
    result = parse_repository_spec(spec_text,
                                   resolver=repository_spec_parse.resolve)

    assert_just_keys(result)
    assert_keys(result, name='some_content', version=None, scm=None, src='some_content')


def test_parse_repository_spec_src():
    spec_text = 'some_namespace.some_content'
    result = parse_repository_spec(spec_text)

    assert_just_keys(result)
    assert_keys(result, namespace='some_namespace', name='some_content',
                version=None, scm=None, src='some_namespace.some_content')


def test_parse_repository_spec_src_version():
    spec_text = 'some_namespace.some_content,1.0.0'
    result = parse_repository_spec(spec_text)

    assert_just_keys(result)
    assert_keys(result, namespace='some_namespace', name='some_content',
                version='1.0.0', scm=None, src='some_namespace.some_content')


def test_parse_repository_spec_src_version_name():
    spec_text = 'some_namespace.some_content,1.2.3,somename'
    result = parse_repository_spec(spec_text)

    assert_just_keys(result)
    assert_keys(result, namespace='some_namespace', name='somename',
                version='1.2.3', scm=None, src='some_namespace.some_content')


def test_parse_repository_spec_src_key_value():
    spec_text = 'src=some_namespace.some_content'
    result = parse_repository_spec(spec_text)

    assert_just_keys(result)
    assert_keys(result, namespace='some_namespace', name='some_content',
                version=None, scm=None, src='some_namespace.some_content')


def test_parse_repository_spec_src_version_key_value():
    spec_text = 'some_namespace.some_content,version=1.0.0'
    result = parse_repository_spec(spec_text)

    assert_just_keys(result)
    assert_keys(result, namespace='some_namespace', name='some_content',
                version='1.0.0', scm=None, src='some_namespace.some_content')


def test_parse_repository_spec_src_version_name_key_value():
    spec_text = 'some_namespace.some_content,1.2.3,name=somename'
    result = parse_repository_spec(spec_text)

    assert_just_keys(result)
    assert_keys(result, namespace='some_namespace', name='somename',
                version='1.2.3', scm=None, src='some_namespace.some_content')


def test_parse_repository_spec_src_version_name_something_invalid_key_value():
    spec_text = 'some_content,1.0.0,name=some_name,foo=bar,some_garbage'
    try:
        parse_repository_spec(spec_text)
    except exceptions.GalaxyClientError:
        return

    assert False, 'spec_text="%s" should have caused a GalaxyClientError' % spec_text




import logging

import attr

log = logging.getLogger(__name__)

# TODO: use 'attr' or similar


@attr.s(frozen=True)
class GalaxyNamespace(object):
    namespace = attr.ib()
    path = attr.ib(default=None, cmp=False)




import logging
import mock

from ansible_galaxy.actions import info

log = logging.getLogger(__name__)


def display_callback(msg, **kwargs):
    log.debug(msg)


def test_info_empty(galaxy_context):
    ret = info.info_repository_specs(galaxy_context,
                                     # mock api
                                     mock.Mock(),
                                     [],
                                     display_callback=display_callback,
                                     offline=True)

    log.debug('ret: %s', ret)


def test_info(galaxy_context):
    ret = info.info_repository_specs(galaxy_context,
                                     # mock api
                                     mock.Mock(),
                                     # ['namespace.repo.content'],
                                     ['namespace.repo'],
                                     display_callback=display_callback,
                                     offline=True)

    log.debug('ret: %s', ret)


def test_repr_remote_repo():
    data = {'summary_fields':
            {'namespace': {'name': 'some_namespace'},
             'content_objects': [{'name': 'some_content',
                                  'content_type': 'role',
                                  'description': 'Description of some content object'}],
             'versions': [],
             },
            'name': 'some_name',
            'description': 'Description of some remote repo',
            'clone_url': 'http://github.com/alikins/some_repo',
            'format': 'multi',
            }
    res = info._repr_remote_repo(data)
    log.debug('res: %s', res)

    assert 'Description of some remote repo'in res


def test_repr_role_info():
    data = {'name': 'some_role',
            'description': 'Description of some_role',
            'some_other_info': 'foo bar',
            'some_dict': {'polarity': 'negative',
                          'description': 'another description?',
                          'coolness': 'not at all'},
            }
    res = info._repr_role_info(data)

    log.debug('res: %s', res)

    assert 'some_role' in res
    assert 'Description of some_role' in res



import logging

from ansible_galaxy_cli.logger import setup

log = logging.getLogger(__name__)


def test_load_config_yaml_no_such_file():
    filename = '/dev/null/doesnotexist.yml'

    res = setup.load_config_yaml(filename)

    # load_config_yaml doesnt throw an exception for missing file
    assert res is None


BAD_YAML = '''
---
stuff: blip
things:
    - notvalidyaml
'''


def test_setup_empty():
    setup.setup({'version': 1})


def test_setup_default(mocker):
    mocker.patch('ansible_galaxy_cli.logger.setup.load_config_yaml',
                 return_value=None)
    setup.setup_default()

    cli_logger = logging.getLogger('ansible_galaxy_cli')

    assert cli_logger.getEffectiveLevel() == logging.INFO

    # should have null handlers at least
    # no hasHandlers for py2.7 so just
    if hasattr(cli_logger, 'hasHandlers'):
        assert cli_logger.hasHandlers()




import logging
import os
import shutil
import tarfile
import tempfile
import pprint

from ansible_galaxy import archive
from ansible_galaxy import exceptions


log = logging.getLogger(__name__)

TMP_PREFIX = 'tmp_mazer_test_archive_'


def test_extract_file_empty():
    tmp_tar = tempfile.TemporaryFile(prefix=TMP_PREFIX)
    # TODO: replace with tmpdir fixture
    tmp_dir = tempfile.mkdtemp(prefix=TMP_PREFIX)

    tar_file = tarfile.TarFile(
                               # name='some-top-level',
                               mode='w',
                               fileobj=tmp_tar)

    log.debug('tar_file.members: %s', tar_file.members)

    tar_file.close()

    # for pathname in tar_example1:
    pathname = 'this_should_not_be_in_empty_tar.yml'
    file_to_extract = {'archive_member': tarfile.TarInfo(name='foo'),
                       'dest_dir': tmp_dir,
                       'dest_filename': pathname,
                       'force_overwrite': True}

    log.debug('files_to_extract: %s', pprint.pformat(file_to_extract))

    read_tar_file = tarfile.TarFile.open(fileobj=tmp_tar, mode='r')

    try:
        res = archive.extract_file(read_tar_file, file_to_extract=file_to_extract)
        log.debug('res: %s', res)
        log.debug('listdir(%s): %s', tmp_dir, os.listdir(tmp_dir))
    except exceptions.GalaxyArchiveError as e:
        log.exception(e)
        return
    except tarfile.ReadError as e:
        log.exception(e)
        return
    finally:
        tmp_tar.close()
        read_tar_file.close()
        shutil.rmtree(tmp_dir)


def test_extract_file_foo():
    tmp_tar_fo = tempfile.NamedTemporaryFile(delete=False, prefix='dddddddddddddd')
    tmp_member_fo = tempfile.NamedTemporaryFile(delete=False, prefix='cccccccccc')
    # TODO: replace with tmpdir fixture
    tmp_dir = tempfile.mkdtemp(prefix='fffffcccc')

    tar_file = tarfile.TarFile.open(mode='w',
                                    fileobj=tmp_tar_fo)
    # fileobj=tmp_tar)

    files_to_extract = []

    pathname = 'foo_blip.yml'
    # for pathname in tar_example1:
    member = tarfile.TarInfo(pathname)
    tar_file.addfile(member, tmp_member_fo)

    log.debug('tar_file2 members: %s', tar_file.getmembers())
    tar_file.close()

    item = {'archive_member': member,
            'dest_dir': tmp_dir,
            'dest_filename': pathname,
            'force_overwrite': False}

    files_to_extract.append(item)
    log.debug('files_to_extract: %s', pprint.pformat(files_to_extract))

    read_tar_file = tarfile.TarFile.open(name=tmp_tar_fo.name, mode='r')
    res = archive.extract_file(read_tar_file, file_to_extract=item)
    log.debug('res: %s', res)

    log.debug('listdir: %s', os.listdir(tmp_dir))
    assert pathname in os.listdir(tmp_dir)

    os.unlink(tmp_tar_fo.name)
    read_tar_file.close()
    tmp_tar_fo.close()
    tmp_member_fo.close()
    os.unlink(tmp_member_fo.name)
    shutil.rmtree(tmp_dir)


def test_extract_files():
    tmp_tar_fo = tempfile.NamedTemporaryFile(delete=False, prefix='dddddddddddddd')
    tmp_member_fo = tempfile.NamedTemporaryFile(delete=False, prefix='cccccccccc')
    # TODO: replace with tmpdir fixture
    tmp_dir = tempfile.mkdtemp(prefix='fffffcccc')

    tar_file = tarfile.TarFile.open(mode='w',
                                    fileobj=tmp_tar_fo)
    # fileobj=tmp_tar)

    files_to_extract = []

    # for pathname in tar_example1:
    pathnames = ['foo_blip.yml',
                 'bar_foo.yml']

    for pathname in pathnames:
        member = tarfile.TarInfo(pathname)
        tar_file.addfile(member, tmp_member_fo)

        item = {'archive_member': member,
                'dest_dir': tmp_dir,
                'dest_filename': pathname,
                'force_overwrite': False}

        files_to_extract.append(item)

    log.debug('tar_file2 members: %s', tar_file.getmembers())
    tar_file.close()

    log.debug('files_to_extract: %s', pprint.pformat(files_to_extract))

    read_tar_file = tarfile.TarFile.open(name=tmp_tar_fo.name, mode='r')
    res_gen = archive.extract_files(read_tar_file, files_to_extract=files_to_extract)
    res = list(res_gen)
    log.debug('res: %s', res)

    log.debug('listdir: %s', os.listdir(tmp_dir))
    assert pathname in os.listdir(tmp_dir)

    os.unlink(tmp_tar_fo.name)
    read_tar_file.close()
    tmp_tar_fo.close()
    tmp_member_fo.close()
    os.unlink(tmp_member_fo.name)
    shutil.rmtree(tmp_dir)


def test_extract_file(mocker):
    # FIXME: rm out of tests tar file example
    # FIXME: generate a test tarfile
    tar_file_mock = mocker.MagicMock(spec_set=tarfile.TarFile)
    # TODO: replace with tmpdir fixture
    tmp_dir = tempfile.mkdtemp(prefix=TMP_PREFIX)

    files_to_extract = []
    # for pathname in tar_example1:
    pathname = 'roles/test-role-d/handlers/main.yml'
    top_dir = 'ansible-content-archive'

    dest_dir = os.path.join(tmp_dir, 'extracted_stuff')

    member = tarfile.TarInfo(os.path.join(top_dir, pathname))

    item = {'archive_member': member,
            'dest_dir': dest_dir,
            'dest_filename': pathname,
            'force_overwrite': True}

    log.debug('files_to_extract: %s', pprint.pformat(files_to_extract))

    res = archive.extract_file(tar_file_mock, file_to_extract=item)

    log.debug('tar_file_mock: %s', tar_file_mock)
    log.debug('tar_file_mock.call_args_list: %s', tar_file_mock.call_args_list)
    log.debug('tar_file_mock.method_calls: %s', tar_file_mock.method_calls)
    log.debug('res: %s', res)

    assert pathname in res
    # log.debug('%s contents: %s', tmp_dir, glob.glob(dest_dir, '**', recursive=True))
    shutil.rmtree(tmp_dir)


def test_extract_file_exists(mocker):
    tar_file_mock = mocker.MagicMock(spec_set=tarfile.TarFile)
    # TODO: replace with tmpdir fixture
    tmp_dir = tempfile.mkdtemp(prefix=TMP_PREFIX)

    files_to_extract = []
    # for pathname in tar_example1:
    pathname = 'roles/test-role-d/handlers/main.yml'
    top_dir = 'ansible-content-archive'

    dest_dir = os.path.join(tmp_dir, 'extracted_stuff')

    def faux_exists(path):
        if path.startswith(dest_dir):
            return True
        return False

    mocker.patch('ansible_galaxy.archive.os.path.exists',
                 side_effect=faux_exists)

    member = tarfile.TarInfo(os.path.join(top_dir, pathname))

    item = {'archive_member': member,
            'dest_dir': dest_dir,
            'dest_filename': pathname,
            'force_overwrite': False}

    log.debug('files_to_extract: %s', pprint.pformat(files_to_extract))

    try:
        archive.extract_file(tar_file_mock, file_to_extract=item)
    except exceptions.GalaxyClientError as e:
        log.exception(e)
        return
    finally:
        shutil.rmtree(tmp_dir)

    assert False, 'Expected a GalaxyClientError because the file to extract already exists, but that did not happen'


def test_extract_files_tmp(mocker):
    # FIXME: rm out of tests tar file example
    # FIXME: generate a test tarfile
    # tar_file = tarfile.TarFile.open(name='/tmp/alikins.testing-content.tar.gz',
    #                                mode='r')

    tar_file_mock = mocker.MagicMock(spec_set=tarfile.TarFile)

    # TODO: replace with tmpdir fixture
    tmp_dir = tempfile.mkdtemp(prefix=TMP_PREFIX)

    files_to_extract = []
    # for pathname in tar_example1:
    top_dir = 'ansible-content-archive'

    dest_dir = os.path.join(tmp_dir, 'extracted_stuff')

    path_list = ['roles/test-role-d/handlers/main.yml',
                 'roles/test-role-a/handlers/main.yml']

    for pathname in path_list:
        member_path = os.path.join(top_dir, pathname)
        member = tarfile.TarInfo(member_path)

        item = {'archive_member': member,
                'dest_dir': dest_dir,
                'dest_filename': pathname,
                'force_overwrite': True}

        files_to_extract.append(item)

    log.debug('files_to_extract: %s', pprint.pformat(files_to_extract))

    res = archive.extract_files(tar_file_mock, files_to_extract=files_to_extract)

    log.debug('tar_file_mock: %s', tar_file_mock)
    log.debug('tar_file_mock.call_args_list: %s', tar_file_mock.call_args_list)
    log.debug('tar_file_mock.method_calls: %s', tar_file_mock.method_calls)
    log.debug('res: %s', list(res))
    # log.debug('%s contents: %s', tmp_dir, glob.glob(dest_dir, '**', recursive=True))
    log.debug('%s contents: %s', tmp_dir, list(os.walk(dest_dir)))
    shutil.rmtree(tmp_dir)


foo = {'content_archive_type': 'multi-content',
       'content_type': None,
       'content_type_requires_meta': True,
       'display_callback': None,
       'extract_to_path': '/home/adrian/.ansible/content',
       'file_name': None,
       'force_overwrite': True,
       'install_all_content': False,
       'install_content_type': 'module',
       'parent_dir': None,
       }


tar_example1 = [
    'ansible-testing-content-master/.gitignore',
    'ansible-testing-content-master/LICENSE',
    'ansible-testing-content-master/README.md',
    'ansible-testing-content-master/action_plugins',
    'ansible-testing-content-master/action_plugins/add_host.py',
    'ansible-testing-content-master/filter_plugins',
    'ansible-testing-content-master/filter_plugins/json_query.py',
    'ansible-testing-content-master/filter_plugins/mathstuff.py',
    'ansible-testing-content-master/library',
    'ansible-testing-content-master/library/elasticsearch_plugin.py',
    'ansible-testing-content-master/library/kibana_plugin.py',
    'ansible-testing-content-master/library/mysql_db.py',
    'ansible-testing-content-master/library/mysql_replication.py',
    'ansible-testing-content-master/library/mysql_user.py',
    'ansible-testing-content-master/library/mysql_variables.py',
    'ansible-testing-content-master/library/redis.py',
    'ansible-testing-content-master/library/riak.py',
    'ansible-testing-content-master/module_utils',
    'ansible-testing-content-master/module_utils/common.py',
    'ansible-testing-content-master/module_utils/helper.py',
    'ansible-testing-content-master/module_utils/inventory.py',
    'ansible-testing-content-master/module_utils/lookup.py',
    'ansible-testing-content-master/module_utils/raw.py',
    'ansible-testing-content-master/module_utils/scale.py',
    'ansible-testing-content-master/roles',
    'ansible-testing-content-master/roles/ansible-role-foobar',
    'ansible-testing-content-master/roles/ansible-role-foobar/.travis.yml',
    'ansible-testing-content-master/roles/ansible-role-foobar/README.md',
    'ansible-testing-content-master/roles/ansible-role-foobar/defaults',
    'ansible-testing-content-master/roles/ansible-role-foobar/defaults/main.yml',
    'ansible-testing-content-master/roles/ansible-role-foobar/handlers',
    'ansible-testing-content-master/roles/ansible-role-foobar/handlers/main.yml',
    'ansible-testing-content-master/roles/ansible-role-foobar/meta',
    'ansible-testing-content-master/roles/ansible-role-foobar/meta/main.yml',
    'ansible-testing-content-master/roles/ansible-role-foobar/tasks',
    'ansible-testing-content-master/roles/ansible-role-foobar/tasks/main.yml',
    'ansible-testing-content-master/roles/ansible-role-foobar/tests',
    'ansible-testing-content-master/roles/ansible-role-foobar/tests/inventory',
    'ansible-testing-content-master/roles/ansible-role-foobar/tests/test.yml',
    'ansible-testing-content-master/roles/ansible-role-foobar/vars',
    'ansible-testing-content-master/roles/ansible-role-foobar/vars/main.yml',
    'ansible-testing-content-master/roles/ansible-test-role-1',
    'ansible-testing-content-master/roles/ansible-test-role-1/.travis.yml',
    'ansible-testing-content-master/roles/ansible-test-role-1/README.md',
    'ansible-testing-content-master/roles/ansible-test-role-1/defaults',
    'ansible-testing-content-master/roles/ansible-test-role-1/defaults/main.yml',
    'ansible-testing-content-master/roles/ansible-test-role-1/handlers',
    'ansible-testing-content-master/roles/ansible-test-role-1/handlers/main.yml',
    'ansible-testing-content-master/roles/ansible-test-role-1/meta',
    'ansible-testing-content-master/roles/ansible-test-role-1/meta/main.yml',
    'ansible-testing-content-master/roles/ansible-test-role-1/tasks',
    'ansible-testing-content-master/roles/ansible-test-role-1/tasks/main.yml',
    'ansible-testing-content-master/roles/ansible-test-role-1/tests',
    'ansible-testing-content-master/roles/ansible-test-role-1/tests/inventory',
    'ansible-testing-content-master/roles/ansible-test-role-1/tests/test.yml',
    'ansible-testing-content-master/roles/ansible-test-role-1/vars',
    'ansible-testing-content-master/roles/ansible-test-role-1/vars/main.yml',
    'ansible-testing-content-master/roles/test-role-a',
    'ansible-testing-content-master/roles/test-role-a/defaults',
    'ansible-testing-content-master/roles/test-role-a/defaults/main.yml',
    'ansible-testing-content-master/roles/test-role-a/handlers',
    'ansible-testing-content-master/roles/test-role-a/handlers/main.yml',
    'ansible-testing-content-master/roles/test-role-a/meta',
    'ansible-testing-content-master/roles/test-role-a/meta/main.yml',
    'ansible-testing-content-master/roles/test-role-a/tasks',
    'ansible-testing-content-master/roles/test-role-a/tasks/main.yml',
    'ansible-testing-content-master/roles/test-role-a/tests',
    'ansible-testing-content-master/roles/test-role-a/tests/inventory',
    'ansible-testing-content-master/roles/test-role-a/tests/test.yml',
    'ansible-testing-content-master/roles/test-role-a/vars',
    'ansible-testing-content-master/roles/test-role-a/vars/main.yml',
    'ansible-testing-content-master/roles/test-role-b',
    'ansible-testing-content-master/roles/test-role-b/.travis.yml',
    'ansible-testing-content-master/roles/test-role-b/README.md',
    'ansible-testing-content-master/roles/test-role-b/defaults',
    'ansible-testing-content-master/roles/test-role-b/defaults/main.yml',
    'ansible-testing-content-master/roles/test-role-b/handlers',
    'ansible-testing-content-master/roles/test-role-b/handlers/main.yml',
    'ansible-testing-content-master/roles/test-role-b/meta',
    'ansible-testing-content-master/roles/test-role-b/meta/main.yml',
    'ansible-testing-content-master/roles/test-role-b/tasks',
    'ansible-testing-content-master/roles/test-role-b/tasks/main.yml',
    'ansible-testing-content-master/roles/test-role-b/tests',
    'ansible-testing-content-master/roles/test-role-b/tests/inventory',
    'ansible-testing-content-master/roles/test-role-b/tests/test.yml',
    'ansible-testing-content-master/roles/test-role-b/vars',
    'ansible-testing-content-master/roles/test-role-b/vars/main.yml',
    'ansible-testing-content-master/roles/test-role-c',
    'ansible-testing-content-master/roles/test-role-c/.travis.yml',
    'ansible-testing-content-master/roles/test-role-c/README.md',
    'ansible-testing-content-master/roles/test-role-c/defaults',
    'ansible-testing-content-master/roles/test-role-c/defaults/main.yml',
    'ansible-testing-content-master/roles/test-role-c/handlers',
    'ansible-testing-content-master/roles/test-role-c/handlers/main.yml',
    'ansible-testing-content-master/roles/test-role-c/meta',
    'ansible-testing-content-master/roles/test-role-c/meta/main.yml',
    'ansible-testing-content-master/roles/test-role-c/tasks',
    'ansible-testing-content-master/roles/test-role-c/tasks/main.yml',
    'ansible-testing-content-master/roles/test-role-c/tests',
    'ansible-testing-content-master/roles/test-role-c/tests/inventory',
    'ansible-testing-content-master/roles/test-role-c/tests/test.yml',
    'ansible-testing-content-master/roles/test-role-c/vars',
    'ansible-testing-content-master/roles/test-role-c/vars/main.yml',
    'ansible-testing-content-master/roles/test-role-d',
    'ansible-testing-content-master/roles/test-role-d/.travis.yml',
    'ansible-testing-content-master/roles/test-role-d/README.md',
    'ansible-testing-content-master/roles/test-role-d/defaults',
    'ansible-testing-content-master/roles/test-role-d/defaults/main.yml',
    'ansible-testing-content-master/roles/test-role-d/handlers',
    'ansible-testing-content-master/roles/test-role-d/handlers/main.yml',
    'ansible-testing-content-master/roles/test-role-d/meta',
    'ansible-testing-content-master/roles/test-role-d/meta/main.yml',
    'ansible-testing-content-master/roles/test-role-d/tasks',
    'ansible-testing-content-master/roles/test-role-d/tasks/main.yml',
    'ansible-testing-content-master/roles/test-role-d/tests',
    'ansible-testing-content-master/roles/test-role-d/tests/inventory',
    'ansible-testing-content-master/roles/test-role-d/tests/test.yml',
    'ansible-testing-content-master/roles/test-role-d/vars',
    'ansible-testing-content-master/roles/test-role-d/vars/main.yml',
    'ansible-testing-content-master/strategy_plugins',
    'ansible-testing-content-master/strategy_plugins/debug.py',
    'ansible-testing-content-master/strategy_plugins/free.py',
    'ansible-testing-content-master/strategy_plugins/linear.py']


def _tar_members(file_list):
    members = []
    for file_name in file_list:
        members.append(tarfile.TarInfo(name=file_name))

    return members



import logging
import os

import pytest

from ansible_galaxy_cli.cli import galaxy
from ansible_galaxy_cli import exceptions as cli_exceptions

log = logging.getLogger(__name__)

COLLECTION_INFO1 = '''
name: some_namespace.some_name
version: 3.1.4
license: GPL-3.0-or-later
description: 'a thing'
'''


def test_build_no_args():
    cli = galaxy.GalaxyCLI(args=['build'])
    cli.parse()

    log.debug('cli.options: %s', cli.options)
    assert cli.options.output_path is None
    assert cli.options.collection_path is None
    assert cli.args == []


def test_build_run_no_args():
    cli = galaxy.GalaxyCLI(args=['build'])
    cli.parse()
    log.debug('cli.options: %s', cli.options)

    rc = cli.run()

    assert rc == os.EX_SOFTWARE


def test_build_run_tmp_collection_path(tmpdir):
    temp_dir = tmpdir.mkdir('mazer_cli_built_run_tmp_output_path_unit_test')
    log.debug('temp_dir: %s', temp_dir.strpath)

    collection_path = tmpdir.mkdir('collection').strpath
    output_path = tmpdir.mkdir('output').strpath

    info_path = os.path.join(collection_path, 'collection_info.yml')
    info_fd = open(info_path, 'w')
    info_fd.write(COLLECTION_INFO1)
    info_fd.close()

    cli = galaxy.GalaxyCLI(args=['mazer',
                                 'build',
                                 '--collection-path', collection_path,
                                 '--output-path', output_path])
    cli.parse()

    log.debug('cli.options: %s', cli.options)

    res = cli.run()
    log.debug('res: %s', res)


def test_info():
    cli = galaxy.GalaxyCLI(args=['info'])
    cli.parse()

    log.debug('cli.options: %s', cli.options)


def test_run_info():
    cli = galaxy.GalaxyCLI(args=['info'])
    cli.parse()
    with pytest.raises(cli_exceptions.CliOptionsError, match="you must specify a user/role name"):
        cli.run()


def test_publish_no_args():
    cli = galaxy.GalaxyCLI(args=['publish'])
    cli.parse()
    with pytest.raises(cli_exceptions.CliOptionsError, match="you must specify a path"):
        cli.run()




# -*- coding: utf-8 -*-
import logging

from ansible_galaxy import exceptions
from ansible_galaxy.utils.text import to_text

log = logging.getLogger(__name__)


def test_galaxy_error():
    exc = exceptions.GalaxyError()
    log.debug('exc: %s', exc)


def test_galaxy_download_error_no_args():
    exc = exceptions.GalaxyDownloadError()
    log.debug('exc: %s', exc)


# Format: byte representation, text representation, encoding of byte representation
VALID_STRINGS = (
    (b'abcde', u'abcde', 'ascii'),
    (b'caf\xc3\xa9', u'caf\xe9', 'utf-8'),
)


def _text_encoding():
    for valid_string, txtrpr, encoding in VALID_STRINGS:
        log.debug('valid_string: %s txtrpr: %s encoding: %s', valid_string, txtrpr, encoding)
        exc = exceptions.GalaxyDownloadError(to_text(valid_string))
        log.debug('exc: %s', exc)
        log.debug(exc)
        log.debug('exc(str): %s', str(exc))
        log.debug('exc(repr): %s', repr(exc))
        ttext = to_text(valid_string)
        log.debug('%s in %s: %s', ttext, '%s' % exc, ttext in '%s' % exc)
        assert ttext in '%s' % exc


def log_text(exc):
    log.debug('exc: %s', exc)
    log.debug(exc)
    log.debug('exc(str): %s', str(exc))
    log.debug('exc(repr): %s', repr(exc))


def _galaxy_download_error(msg, url):
    # url kw but no message
    log.debug('msg: %s %s', msg, type(msg))
    log.debug('url: %s %s', url, type(url))
    exc = exceptions.GalaxyDownloadError(url=url)
    log.debug('exc: %s', exc)
    assert exc.url == url
    assert url in str(exc)

    # multiple args, but no url kwarg
    exc = exceptions.GalaxyDownloadError(url, msg)
    log.debug('exc: %s', exc)
    assert exc.url is None

    # a message arg and a url kwarg
    exc = exceptions.GalaxyDownloadError(msg, url=url)
    log.debug('exc: %s', exc)
    assert exc.url == url
    assert url in str(exc)
    assert msg in str(exc)

    # just a single message arg, no url
    exc = exceptions.GalaxyDownloadError(msg)
    log.debug('exc: %s', exc)
    assert exc.url is None
    assert msg in '%s' % exc
    # str(exc)


def test_galaxy_download_error():
    url = u'http://error.example.com'
    msgs = [u'some_message']

    for msg in msgs:
        _galaxy_download_error(msg, url)



import collections
import logging

from ansible_galaxy import installed_content_item_db
from ansible_galaxy import installed_repository_db
from ansible_galaxy import matchers

log = logging.getLogger(__name__)


def _list(galaxy_context,
          repository_spec_match_filter=None,
          list_content=False,
          display_callback=None):

    log.debug('list_content: %s', list_content)

    all_repository_match = repository_spec_match_filter or matchers.MatchAll()

    # We search for installed repos to list, and then display all the content in those installed repos
    icidb = installed_content_item_db.InstalledContentItemDatabase(galaxy_context)

    irdb = installed_repository_db.InstalledRepositoryDatabase(galaxy_context)

    # accumulate for api return
    repo_list = []

    for installed_repository in irdb.select(repository_spec_match_filter=all_repository_match):
        log.debug('installed_repo: %s', installed_repository)

        content_items = collections.defaultdict(list)

        # Find all the content items for this particular repo
        repository_match = matchers.MatchRepositorySpec([installed_repository.repository_spec])

        for content_info in icidb.select(repository_spec_match_filter=repository_match):
            content_dict = content_info.copy()

            content_item_type = content_dict['content_data'].content_item_type

            # revisit this output format once we get some feedback
            content_dict.update({'type': content_item_type,
                                 'name': content_dict['content_data'].name,
                                 # 'installed_repo_namespace': repo.namespace,
                                 # 'installed_repo_name': repo.name,
                                 # 'installed_repo_path': repo.path,
                                 # 'installed_repo_id': repo.repository_spec.label,
                                 'installed_repository': installed_repository,
                                 })

            content_items[content_item_type].append(content_dict)
            # content_item_list.append(content_dict)

        repo_dict = {'content_items': content_items,
                     'installed_repository': installed_repository}
        repo_list.append(repo_dict)

    for repo_item in repo_list:
        repo_msg = "repo={installed_repository.repository_spec.label}, type=repository, version={installed_repository.repository_spec.version}"
        display_callback(repo_msg.format(**repo_item))

        if not list_content:
            continue

        for content_item_type_key, content_items_data in repo_item['content_items'].items():
            content_msg = "repo={installed_repository.repository_spec.label}, type={type}, name={name}, " + \
                "version={installed_repository.repository_spec.version}"
            # content_msg = "    type={type}, name={name}, " + \
            #    "version={installed_repository.repository_spec.version}"

            # type_msg = "  {content_item_type}:"
            # display_callback(type_msg.format(content_item_type=content_item_type_key))

            log.debug('content_item: %s', content_items_data)
            log.debug('content_item_type_key: %s', content_item_type_key)

            for content_item_data in content_items_data:
                display_callback(content_msg.format(**content_item_data))
        # display_callback(msg.format(**content_dict))

    return repo_list


def list_action(galaxy_context,
                repository_spec_match_filter=None,
                list_content=False,
                display_callback=None):
    '''Run _list action and return an exit code suitable for process exit'''

    _list(galaxy_context,
          repository_spec_match_filter=repository_spec_match_filter,
          list_content=list_content,
          display_callback=display_callback)

    return 0



import logging

from ansible_galaxy import exceptions

log = logging.getLogger(__name__)


def repo_url_to_repo_name(repo_url):
    # gets the role name out of a repo like
    # http://git.example.com/repos/repo.git" => "repo"

    if '://' not in repo_url and '@' not in repo_url:
        return repo_url
    trailing_path = repo_url.split('/')[-1]
    if trailing_path.endswith('.git'):
        trailing_path = trailing_path[:-4]
    if trailing_path.endswith('.tar.gz'):
        trailing_path = trailing_path[:-7]
    if ',' in trailing_path:
        trailing_path = trailing_path.split(',')[0]
    return trailing_path


# TODO: test cases
def parse_repository_name(name):
    "split a full repository_name into namespace, repo_name, content_name"

    repo_name = None
    try:
        parts = name.split(".")
        user_name = parts[0]
        if len(parts) > 2:
            repo_name = parts[1]
            name = '.'.join(parts[2:])
        else:
            name = '.'.join(parts[1:])
    except Exception as e:
        log.exception(e)
        raise exceptions.GalaxyClientError("Invalid content name (%s). Specify content as format: username.contentname" % name)

    return (user_name, repo_name, name)




import logging


# mv details of this here
from ansible_galaxy import exceptions
from ansible_galaxy import download
from ansible_galaxy.fetch import base
# from ansible_galaxy.models.repository_spec import RepositorySpec
from ansible_galaxy.rest_api import GalaxyAPI
from ansible_galaxy import repository_version

log = logging.getLogger(__name__)


def get_download_url(repo_data=None, external_url=None, repoversion=None):
    repo_data = repo_data or {}

    # If we want a specific version, provide an exact match repoversion.
    # if provided an exact match repoversion, use its download_url
    if 'download_url' in repoversion:
        return repoversion['download_url']

    # but then try whatever the Repository suggests for download_url
    # This should be the case if we dont specify a version and the galaxy Repository
    # has no versions associated to it, so this will likely reference the latest in
    # the default branch
    if 'download_url' in repo_data:
        return repo_data['download_url']

    # server response didn't suggest a download_url, take a guess and make one up
    if external_url and repoversion:
        archive_url = '%s/archive/%s.tar.gz' % (external_url, repoversion['version'])
        return archive_url


def select_repository_version(repoversions, version):
    # repoversion's 'version' is 'not null' so should always exist
    # however, the list of repoversions can be empty

    # If the rest api returns a empty list for repo versions, return an
    # empty dict for 'no version'
    if not repoversions:
        return {}

    # we could build a map/dict first and search in it, but we only use this
    # once, so this linear search is ok, since building the map would be that
    # plus the getitem
    results = [x for x in repoversions if x['version'] == version]

    # no matching versions, return an empty dict
    # TODO: raise VersionNotFoundError ? return some sort of NullRepositoryVersion instance?
    if not results:
        return {}

    # repoversions is uniq on (version, repo.id) so for any given repo,
    # there should only be one result here
    repoversion = results.pop()
    return repoversion


# TODO: split into galaxy_role/galaxy_collection ?
class GalaxyUrlFetch(base.BaseFetch):
    fetch_method = 'galaxy_url'

    def __init__(self, galaxy_context, requirement_spec):
        super(GalaxyUrlFetch, self).__init__()

        self.requirement_spec = requirement_spec
        self.galaxy_context = galaxy_context

        self.validate_certs = not self.galaxy_context.server['ignore_certs']

        log.debug('requirement_spec: %s', requirement_spec)
        # log.debug('Validate TLS certificates: %s', self.validate_certs)

    def find(self):
        api = GalaxyAPI(self.galaxy_context)

        namespace = self.requirement_spec.namespace
        repo_name = self.requirement_spec.name

        log.debug('Querying %s for namespace=%s, name=%s', self.galaxy_context.server['url'], namespace, repo_name)

        # TODO: extract parsing of cli content sorta-url thing and add better tests

        # FIXME: exception handling
        repo_data = api.lookup_repo_by_name(namespace, repo_name)

        if not repo_data:
            raise exceptions.GalaxyClientError("- sorry, %s was not found on %s." % (self.requirement_spec.label,
                                                                                     api.api_server))

        # FIXME - Need to update our API calls once Galaxy has them implemented
        related = repo_data.get('related', {})

        repo_versions_url = related.get('versions', None)

        # FIXME: exception handling
        repoversions = api.fetch_content_related(repo_versions_url)

        content_repo_versions = [a.get('version') for a in repoversions if a.get('version', None)]

        repo_version_best = repository_version.get_repository_version(repo_data,
                                                                      requirement_spec=self.requirement_spec,
                                                                      repository_versions=content_repo_versions)

        # get the RepositoryVersion obj (or its data anyway)
        _repoversion = select_repository_version(repoversions, repo_version_best)

        # Note: download_url can point anywhere...
        external_url = repo_data.get('external_url', None)

        if not external_url:
            raise exceptions.GalaxyError('no external_url info on the Repository object from %s' % self.requirement_spec.label)

        results = {'content': {'galaxy_namespace': namespace,
                               'repo_name': repo_name,
                               'version': _repoversion.get('version')},
                   'requirement_spec_version_spec': self.requirement_spec.version_spec,
                   'custom': {'external_url': external_url,
                              'repo_data': repo_data,
                              'repoversion': _repoversion,
                              },
                   }

        return results

    def fetch(self, find_results=None):
        find_results = find_results or {}

        results = {}

        download_url = get_download_url(repo_data=find_results['custom']['repo_data'],
                                        external_url=find_results['custom']['external_url'],
                                        repoversion=find_results['custom']['repoversion'])

        # download_url = _build_download_url(external_url=external_url, version=_content_version)
        # TODO: error handling if there is no download_url

        log.debug('repository_spec=%s', self.requirement_spec)
        log.debug('download_url=%s', download_url)

        # for including in any error messages or logging for this fetch
        self.remote_resource = download_url

        # can raise GalaxyDownloadError
        repository_archive_path = download.fetch_url(download_url,
                                                     validate_certs=self.validate_certs)

        self.local_path = repository_archive_path

        log.debug('repository_archive_path=%s', repository_archive_path)

        # TODO: This is indication that a fetcher is wrong abstraction. A fetch
        #       can resolve a name/spec, find metadata about the content including avail versions,
        #       compare/sort versions, select matching versions, find a download uri, and finally
        #       actually fetch it.
        #       Ie, more of a RepositoryRepository (aiee) (RepositorySource? RepositoryChannel? RepositoryProvider?)
        #       that is a remote 'channel' with info and content itself.
        results = {'archive_path': repository_archive_path,
                   'download_url': download_url,
                   'fetch_method': self.fetch_method}

        results['custom'] = {}
        results['content'] = find_results['content']
        results['content']['fetched_version'] = find_results['custom']['repoversion'].get('version')

        return results



from __future__ import print_function

# if this looks a lot like reimplement 'logging', that is
# true...

import logging
import sys

log = logging.getLogger(__name__)

INFO_LEVEL = {
    'prefix': '',
    'log_level': logging.INFO,
    'stream': sys.stdout,
}

DISPLAY_LEVEL_MAP = {
    'warning': {
        'prefix': 'WARNING| ',
        'log_level': logging.WARNING,
        'stream': sys.stderr
    },
    'info': INFO_LEVEL
}


def stdout_display_callback(*args, **kwargs):
    level_arg = kwargs.get('level', None)
    level_prefix = DISPLAY_LEVEL_MAP.get(level_arg, INFO_LEVEL)['prefix']
    level_stream = DISPLAY_LEVEL_MAP.get(level_arg, INFO_LEVEL)['stream']

    print('%s%s' % (level_prefix, ''.join(args)), file=level_stream)


# will log whatever is display with display callback to the ansible_galaxy.display logger
def log_display_callback(*args, **kwargs):
    level_arg = kwargs.get('level', None)
    # find custom level, otherwise use INFO_LEVELs
    log_level = DISPLAY_LEVEL_MAP.get(level_arg, INFO_LEVEL)['log_level']
    # log.log(log_level, 'DISPLAY: %s', ''.join(args), extra={'display_args': args})
    log.log(log_level, '%s', ''.join(args), extra={'display_args': args})


def display_callback(*args, **kwargs):
    stdout_display_callback(*args, **kwargs)
    log_display_callback(*args, **kwargs)


def null_display_callback(*args, **kwargs):
    pass



import logging
import operator

import attr
import pytest
import six

from ansible_galaxy.models import repository
from ansible_galaxy.models import repository_spec

log = logging.getLogger(__name__)


@pytest.fixture
def csf(request):
    _cs = repository_spec.RepositorySpec(namespace='somenamespace', name='somename', version='1.2.3')
    yield _cs


def test_frozen(csf):
    cr = repository.Repository(repository_spec=csf)

    new_cr = repository_spec.RepositorySpec(namespace='somenamespace', name='somename', version='1.2.3')

    with pytest.raises(attr.exceptions.FrozenInstanceError):
        cr.repository_spec = new_cr

    with pytest.raises(attr.exceptions.FrozenInstanceError):
        cr.path = '/dev/null/somepath'

    with pytest.raises(attr.exceptions.FrozenInstanceError):
        cr.installed = True


def test_cr_init_no_args():
    with pytest.raises(TypeError) as exc_info:
        repository.Repository()

    log.debug('exc_info.getrepr(): %s', exc_info.getrepr(showlocals=True, style='native'))

    # different TypeError strings for py2 vs py3
    if six.PY3:
        assert exc_info.match("__init__.*missing.*1.*'repository_spec'")

    if six.PY2:
        # __init__() takes at least 2 arguments (1 given)
        assert exc_info.match("__init__.*takes at least.*arguments.*given")


def test_cr_init(csf):
    cr = repository.Repository(repository_spec=csf)

    assert isinstance(cr, repository.Repository)
    assert 'Repository' in repr(cr)


def test_cr_init_path(csf):
    path = '/dev/null/wherever'
    cr = repository.Repository(repository_spec=csf,
                               path=path)

    assert isinstance(cr, repository.Repository)
    assert path in repr(cr)
    assert cr.path == path


def CR(repository_spec=None, path=None):
    if path:
        return repository.Repository(repository_spec=repository_spec, path=path)
    return repository.Repository(repository_spec=repository_spec)


ns_n = repository_spec.RepositorySpec(namespace='ns', name='n', version='1.0.0')
diffns_n = repository_spec.RepositorySpec(namespace='diffns', name='n', version='1.0.0')
ns_n_1_0_0 = ns_n
diffns_n_1_0_0 = diffns_n
ns_n_1_0_1 = repository_spec.RepositorySpec(namespace='ns', name='n', version='1.0.1')


path1 = '/dev/null/1'
path2 = '/dev/null/2'


@pytest.mark.parametrize("left,right,op,expected",
                         [
                             (CR(ns_n), CR(ns_n), operator.eq, True),
                             (CR(ns_n), CR(ns_n), operator.ne, False),
                             (CR(ns_n), CR(ns_n), operator.le, True),
                             (CR(ns_n), CR(ns_n), operator.ge, True),
                             (CR(ns_n), CR(ns_n), operator.is_, False),
                             (CR(ns_n), CR(diffns_n), operator.eq, False),
                             (CR(ns_n), CR(diffns_n), operator.le, False),
                             (CR(ns_n, path=path1), CR(ns_n, path=path1), operator.eq, True),
                             (CR(ns_n, path=path1), CR(ns_n, path=path1), operator.ne, False),
                             (CR(ns_n, path=path1), CR(ns_n, path=path2), operator.eq, False),
                             (CR(ns_n, path=path1), CR(ns_n, path=path2), operator.ne, True),
                             (CR(ns_n, path=path1), CR(ns_n_1_0_1, path=path1), operator.eq, False),
                             (CR(ns_n, path=path1), CR(ns_n_1_0_1, path=path1), operator.ne, True),
                             # (CR(ns_n), CR(diffns_n), operator.ge, False),
                             # (CR(ns_n), CR(diffns_n), operator.gt, False),
                             # (CR(ns_n), CR(diffns_n), operator.lt, False),

                          ])
def test_cr_equality_ops(left, right, op, expected):
    log.debug('left:%s', left)
    log.debug('right: %s', right)
    log.debug('op: %s', op)
    log.debug('expected: %s', expected)
    res = op(left, right)
    log.debug('res: %s', res)
    import pprint
    log.debug('left dict: %s', pprint.pformat(list(left.__attrs_attrs__)))
    assert res == expected


@pytest.mark.parametrize("left,op,expected",
                         [
                             (CR(ns_n), operator.truth, True),
                             (CR(ns_n), operator.not_, False),
                             (CR(ns_n), operator.truth, True),
                             (CR(ns_n), operator.not_, False),
                          ])
def test_cr_bool_ops(left, op, expected):
    log.debug('left: %s, op: %s expected: %s', left, op, expected)
    res = op(left)
    log.debug('res: %s', res)
    assert res == expected



import logging
import os

from ansible_galaxy import galaxy_repository_spec
from ansible_galaxy import repository_spec_parse
from ansible_galaxy import exceptions

from ansible_galaxy.models.repository_spec import RepositorySpec, FetchMethods


log = logging.getLogger(__name__)


def is_scm(repository_spec_string):
    if '://' in repository_spec_string or '@' in repository_spec_string:
        return True

    return False


def choose_repository_fetch_method(repository_spec_string, editable=False):
    log.debug('repository_spec_string: %s', repository_spec_string)

    if is_scm(repository_spec_string):
        # create tar file from scm url
        return FetchMethods.SCM_URL

    comma_parts = repository_spec_string.split(',', 1)
    potential_filename = comma_parts[0]
    fetch_method = None
    if editable and os.path.isdir(potential_filename):
        fetch_method = FetchMethods.EDITABLE
    elif os.path.isfile(potential_filename):
        # installing a local tar.gz
        fetch_method = FetchMethods.LOCAL_FILE
    elif '://' in repository_spec_string:
        fetch_method = FetchMethods.REMOTE_URL
    elif '.' in repository_spec_string and len(repository_spec_string.split('.', 1)) == 2:
        fetch_method = FetchMethods.GALAXY_URL
    else:
        msg = ('Failed to determine fetch method for content spec %s. '
               'Expecting a Galaxy name, SCM path, remote URL, path to a local '
               'archive file, or -e option and a directory path' % repository_spec_string)
        raise exceptions.GalaxyError(msg)
    return fetch_method


def resolve(data):
    src = data['src']
    if data['name'] is None:
        scm_name = repository_spec_parse.repo_url_to_repo_name(src)
        data['name'] = scm_name
        if '+' in src:
            (scm_url, scm_src) = src.split('+', 1)
            data['scm'] = scm_url
            data['src'] = scm_src

    # split the name on '.' and recombine the first 1 or 2
    name_parts = data['name'].split('.')
    new_name_parts = []
    new_name_parts.append(name_parts.pop(0))

    # we may not have a second part to the name
    try:
        new_name_parts.append(name_parts.pop(0))
    except IndexError:
        pass

    # combine the name parts, which may be one or two parts
    data['name'] = '.'.join(new_name_parts)

    return data


def editable_resolve(data):
    log.debug('data: %s', data)

    src = data['src']
    if src.startswith('/'):
        dir_name = os.path.basename(os.path.normpath(src))
        log.debug('dir_name: %s', dir_name)
        data['name'] = dir_name
    return data


def spec_data_from_string(repository_spec_string, namespace_override=None, fetch_method=None, editable=False):
    fetch_method = choose_repository_fetch_method(repository_spec_string, editable=editable)

    spec_data = repository_spec_parse.parse_string(repository_spec_string)
    spec_data['fetch_method'] = fetch_method

    log.debug('spec_data: %s', spec_data)

    resolved_data = {}

    if fetch_method == FetchMethods.GALAXY_URL:
        resolved_data = galaxy_repository_spec.resolve(spec_data)
    elif fetch_method == FetchMethods.EDITABLE:
        resolved_data = editable_resolve(spec_data)
    else:
        resolved_data = resolve(spec_data)

    log.debug('resolved_data: %s', resolved_data)
    spec_data.update(resolved_data)

    if namespace_override:
        if spec_data.get('namespace'):
            log.debug('using --namespace provided namespace "%s" to override detected namespace "%s"',
                      namespace_override,
                      spec_data['namespace'])
        else:
            log.debug('using --namespace provided namespace "%s" to since there was no namespace in "%s"',
                      namespace_override,
                      repository_spec_string)

        spec_data['namespace'] = namespace_override

    return spec_data


def repository_spec_from_string(repository_spec_string, namespace_override=None, editable=False):
    spec_data = spec_data_from_string(repository_spec_string, namespace_override=namespace_override, editable=editable)

    log.debug('spec_data: %s', spec_data)

    return RepositorySpec(name=spec_data.get('name'),
                          namespace=spec_data.get('namespace'),
                          version=spec_data.get('version'),
                          # version=version,
                          scm=spec_data.get('scm'),
                          spec_string=spec_data.get('spec_string'),
                          fetch_method=spec_data.get('fetch_method'),
                          src=spec_data.get('src'))



import logging
import os

import yaml

from ansible_galaxy.models.install_info import InstallInfo

log = logging.getLogger(__name__)


def load(data_or_file_object):
    # log.debug('loading content install info from %s', getattr(data_or_file_object, 'name', data_or_file_object))

    info_dict = yaml.safe_load(data_or_file_object)

    # an empty .galaxy_install_info
    if info_dict is None:
        return None

    # log.debug('info_dict: %s', info_dict)
    install_info = InstallInfo(version=info_dict.get('version', None),
                               install_date=info_dict.get('install_date', None),
                               install_date_iso=info_dict.get('install_date_iso', None))

    # log.debug('install_info loaded from %s', install_info)
    return install_info


def load_from_filename(filename):
    if not os.path.isfile(filename):
        return None

    try:
        f = open(filename, 'r')
        return load(f)
    except Exception as e:
        log.exception(e)
        log.debug('Unable to load install info from path: %s', filename)
        return False
    finally:
        f.close()


def save(install_info_dict, filename):
    # log.debug('saving install info to %s', filename)

    if not os.path.exists(os.path.dirname(filename)):
        os.makedirs(os.path.dirname(filename))

    with open(filename, 'w+') as f:
        # FIXME: just return the install_info dict (or better, build it elsewhere and pass in)
        # FIXME: stop minging self state
        try:
            yaml.safe_dump(install_info_dict, f, default_flow_style=False)
        except Exception as e:
            log.warning('unable to serialize .galaxy_install_info to filename=%s for data=%s', filename, install_info_dict)
            log.exception(e)
            return False

    log.debug('wrote galaxy_install_info to %s', filename)
    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('CHANGELOG.rst') as changelog_file:
    changelog = changelog_file.read()

requirements = ['six',
                'PyYaml',
                'jinja2',
                'semantic_version',
                'yamlloader',
                # used for data classes
                # 18.1.0 introduces the 'factory' keyword
                'attrs>=18.1.0',
                ]

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest', ]

entry_points = {
    'console_scripts': ['mazer = ansible_galaxy_cli.__main__:main']
}

setup(
    entry_points=entry_points,
    install_requires=requirements,
    license="GPLv3",
    long_description=readme + '\n\n' + changelog,
    long_description_content_type='text/x-rst',
    include_package_data=True,
    keywords='mazer',
    packages=find_packages(include=['ansible_galaxy', 'ansible_galaxy_cli',
                                    'ansible_galaxy.*', 'ansible_galaxy_cli.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/ansible/galaxy-cli',
    version='0.4.0',
    zip_safe=False,
)



# (c) 2012-2014, Michael DeHaan <michael.dehaan@gmail.com>
# (c) 2016, Toshio Kuratomi <tkuratomi@ansible.com>
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import logging
import operator
import optparse
import re
import six

from abc import ABCMeta, abstractmethod

from ansible_galaxy.config import defaults
from ansible_galaxy import display
from ansible_galaxy.utils.text import to_text
from ansible_galaxy_cli import exceptions as cli_exceptions

log = logging.getLogger(__name__)


class SortedOptParser(optparse.OptionParser):
    '''Optparser which sorts the options by opt before outputting --help'''

    def format_help(self, formatter=None):
        self.option_list.sort(key=operator.methodcaller('get_opt_string'))
        return optparse.OptionParser.format_help(self, formatter=None)


# Note: Inherit from SortedOptParser so that we get our format_help method
class InvalidOptsParser(SortedOptParser):
    '''Ignore invalid options.

    Meant for the special case where we need to take care of help and version
    but may not know the full range of options yet.  (See it in use in set_action)
    '''

    def __init__(self, parser):
        # Since this is special purposed to just handle help and version, we
        # take a pre-existing option parser here and set our options from
        # that.  This allows us to give accurate help based on the given
        # option parser.
        SortedOptParser.__init__(self, usage=parser.usage,
                                 option_list=parser.option_list,
                                 option_class=parser.option_class,
                                 conflict_handler=parser.conflict_handler,
                                 description=parser.description,
                                 formatter=parser.formatter,
                                 add_help_option=False,
                                 prog=parser.prog,
                                 epilog=parser.epilog)
        self.version = parser.version

    def _process_long_opt(self, rargs, values):
        try:
            optparse.OptionParser._process_long_opt(self, rargs, values)
        except optparse.BadOptionError:
            pass

    def _process_short_opts(self, rargs, values):
        try:
            optparse.OptionParser._process_short_opts(self, rargs, values)
        except optparse.BadOptionError:
            pass


class CLI(six.with_metaclass(ABCMeta, object)):
    ''' code behind bin/ansible* programs '''

    VALID_ACTIONS = []
    VALID_ACTION_ALIASES = {}

    _ITALIC = re.compile(r"I\(([^)]+)\)")
    _BOLD = re.compile(r"B\(([^)]+)\)")
    _MODULE = re.compile(r"M\(([^)]+)\)")
    _URL = re.compile(r"U\(([^)]+)\)")
    _CONST = re.compile(r"C\(([^)]+)\)")

    PAGER = 'less'

    # -F (quit-if-one-screen) -R (allow raw ansi control chars)
    # -S (chop long lines) -X (disable termcap init and de-init)
    LESS_OPTS = 'FRSX'
    SKIP_INVENTORY_DEFAULTS = False

    def __init__(self, args, callback=None):
        """
        Base init method for all command line programs
        """

        self.args = args
        self._orig_args_copy = self.args[:]
        self.options = None
        self.parser = None
        self.action = None
        self.callback = callback
        self.log = logging.getLogger(__name__ + '.' + self.__class__.__name__)
        self.config_file_path = None

    def set_action(self):
        """
        Get the action the user wants to execute from the sys argv list.
        """
        for i in range(0, len(self.args)):
            arg = self.args[i]
            if arg in self.VALID_ACTIONS:
                self.action = arg
                del self.args[i]
                break
            if arg in self.VALID_ACTION_ALIASES:
                self.action = self.VALID_ACTION_ALIASES[arg]
                del self.args[i]
                break

        if not self.action:
            # if we're asked for help or version, we don't need an action.
            # have to use a special purpose Option Parser to figure that out as
            # the standard OptionParser throws an error for unknown options and
            # without knowing action, we only know of a subset of the options
            # that could be legal for this command
            tmp_parser = InvalidOptsParser(self.parser)
            tmp_options, _ = tmp_parser.parse_args(self.args)
            if not(hasattr(tmp_options, 'help') and tmp_options.help) or (hasattr(tmp_options, 'version') and tmp_options.version):
                raise cli_exceptions.CliOptionsError("Missing required action")

    def execute(self):
        """
        Actually runs a child defined method using the execute_<action> pattern
        """
        if '-' in self.action:
            # Call a function named execute_some_action if the action is some-action
            fn = getattr(self, "execute_%s" % self.action.replace('-', '_'))
        else:
            fn = getattr(self, "execute_%s" % self.action)
        return fn()

    @abstractmethod
    def run(self):
        """Run the ansible command

        Subclasses must implement this method.  It does the actual work of
        running an Ansible command.
        """
        log.debug('self.args: %s', self.args)

        if self.config_file_path:
            log.info(u"Using %s as config file", to_text(self.config_file_path))
        else:
            log.info(u"No config file found; using defaults")

    def validate_conflicts(self, vault_opts=False, runas_opts=False, fork_opts=False, vault_rekey_opts=False):
        ''' check for conflicting options '''

        op = self.options

        if vault_opts:
            # Check for vault related conflicts
            if (op.ask_vault_pass and op.vault_password_files):
                self.parser.error("--ask-vault-pass and --vault-password-file are mutually exclusive")

        if vault_rekey_opts:
            if (op.new_vault_id and op.new_vault_password_file):
                self.parser.error("--new-vault-password-file and --new-vault-id are mutually exclusive")

        if runas_opts:
            # Check for privilege escalation conflicts
            if ((op.su or op.su_user) and (op.sudo or op.sudo_user) or
                    (op.su or op.su_user) and (op.become or op.become_user) or
                    (op.sudo or op.sudo_user) and (op.become or op.become_user)):

                self.parser.error("Sudo arguments ('--sudo', '--sudo-user', and '--ask-sudo-pass') and su arguments ('--su', '--su-user', and '--ask-su-pass') "
                                  "and become arguments ('--become', '--become-user', and '--ask-become-pass') are exclusive of each other")

        if fork_opts:
            if op.forks < 1:
                self.parser.error("The number of processes (--forks) must be >= 1")

    @staticmethod
    def base_parser(usage="", output_opts=False, runas_opts=False, meta_opts=False, runtask_opts=False, vault_opts=False, module_opts=False,
                    async_opts=False, connect_opts=False, subset_opts=False, check_opts=False, inventory_opts=False, epilog=None, fork_opts=False,
                    runas_prompt_opts=False, desc=None, basedir_opts=False, vault_rekey_opts=False):
        ''' create an options parser for most ansible scripts '''

        # base opts
        parser = SortedOptParser(usage,  description=desc, epilog=epilog)
        parser.add_option('-v', '--verbose', dest='verbosity', default=defaults.VERBOSITY, action="count",
                          help="verbose mode (-vvv for more, -vvvv to enable connection debugging)")

        return parser

    @abstractmethod
    def parse(self):
        """Parse the command line args

        This method parses the command line arguments.  It uses the parser
        stored in the self.parser attribute and saves the args and options in
        self.args and self.options respectively.

        Subclasses need to implement this method.  They will usually create
        a base_parser, add their own options to the base_parser, and then call
        this method to do the actual parsing.  An implementation will look
        something like this::

            def parse(self):
                parser = super(MyCLI, self).base_parser(usage="My Ansible CLI", inventory_opts=True)
                parser.add_option('--my-option', dest='my_option', action='store')
                self.parser = parser
                super(MyCLI, self).parse()
                # If some additional transformations are needed for the
                # arguments and options, do it here.
        """

        self.options, self.args = self.parser.parse_args(self.args[1:])
        # process inventory options except for CLIs that require their own processing

    # FIXME: replace with output callback
    def display(self, *args, **kwargs):
        kwargs.pop('color', None)
        display.display_callback(*args, **kwargs)

    @classmethod
    def tty_ify(cls, text):

        t = cls._ITALIC.sub("`" + r"\1" + "'", text)    # I(word) => `word'
        t = cls._BOLD.sub("*" + r"\1" + "*", t)         # B(word) => *word*
        t = cls._MODULE.sub("[" + r"\1" + "]", t)       # M(word) => [word]
        t = cls._URL.sub(r"\1", t)                      # U(word) => word
        t = cls._CONST.sub("`" + r"\1" + "'", t)        # C(word) => `word'

        return t



import logging
import os

from ansible_galaxy.actions import build
from ansible_galaxy.models.build_context import BuildContext

log = logging.getLogger(__name__)


# FIXME: generalize/share
def display_callback(msg, **kwargs):
    log.debug(msg)


def test_build(galaxy_context, tmpdir):
    output_path = tmpdir.mkdir('mazer_test_build_action_test_build')

    collection_path = os.path.join(os.path.dirname(__file__), '../', 'collection_examples/hello')
    build_context = BuildContext(collection_path, output_path=output_path.strpath)
    ret = build.build(galaxy_context, build_context, display_callback)
    log.debug('ret: %s', ret)

    assert ret == 0, 'build action return code was not 0 but was %s' % ret




import logging

from ansible_galaxy import download
from ansible_galaxy.fetch import base

log = logging.getLogger(__name__)


class RemoteUrlFetch(base.BaseFetch):
    fetch_method = 'remote_url'

    def __init__(self, requirement_spec, validate_certs=True):
        super(RemoteUrlFetch, self).__init__()

        self.requirement_spec = requirement_spec
        self.remote_url = requirement_spec.src

        self.validate_certs = validate_certs
        log.debug('Validate TLS certificates: %s', self.validate_certs)

        self.remote_resource = self.remote_url

    def find(self):
        results = {'content': {'galaxy_namespace': self.requirement_spec.namespace,
                               'repo_name': self.requirement_spec.name},
                   }

        return results

    def fetch(self, find_results=None):
        '''Download the remote_url to a temp file

        Can raise GalaxyDownloadError on any exception while downloadin remote_url and saving it.'''

        find_results = find_results or {}

        # NOTE: could move download.fetch_url here instead of splitting it
        repository_archive_path = download.fetch_url(self.remote_url, validate_certs=self.validate_certs)
        self.local_path = repository_archive_path

        log.debug('repository_archive_path=%s', repository_archive_path)

        results = {'archive_path': repository_archive_path,
                   'fetch_method': self.fetch_method}
        results['content'] = find_results['content']
        results['custom'] = {'remote_url': self.remote_url,
                             'validate_certs': self.validate_certs}
        return results




import logging
import pprint

import yaml

from ansible_galaxy.models.collection_info import \
    CollectionInfo
from ansible_galaxy.models.collection_artifact_file import \
    CollectionArtifactFile
from ansible_galaxy.models.collection_artifact_manifest import \
    CollectionArtifactManifest

log = logging.getLogger(__name__)
pf = pprint.pformat

COLLECTION_MANIFEST_FILENAME = 'MANIFEST.json'


# TODO: replace with a generic version for cases
#       where SomeClass(**dict_from_yaml) works
def load(data_or_file_object):

    # FIXME: This file is json now, so could use the regular json.load()
    #        (this works as well since json is subset of yaml...)
    log.debug('loading collection info from %s',
              pf(data_or_file_object))

    data_dict = yaml.safe_load(data_or_file_object)

    # log.debug('data: %s', pf(data_dict))
    # log.debug('data_dict: %s', data_dict)

    col_info = CollectionInfo(**data_dict['collection_info'])

    file_manifest_file = None
    if data_dict.get('file_manifest_file', None):
        file_manifest_file = CollectionArtifactFile(**data_dict['file_manifest_file'])

    instance = CollectionArtifactManifest(collection_info=col_info,
                                          file_manifest_file=file_manifest_file)

    log.debug('%s instance from_kwargs', type(instance))

    log.debug('collection_artifact_manifest: %r', instance)

    return instance



import logging

import semantic_version
import yaml

from ansible_galaxy.models.requirement import Requirement, RequirementOps, RequirementScopes
from ansible_galaxy.models.requirement_spec import RequirementSpec
from ansible_galaxy.repository_spec import spec_data_from_string
from ansible_galaxy.utils import yaml_parse

log = logging.getLogger(__name__)


def load(data_or_file_object, repository_spec=None):
    log.debug('START of load of requirements %s', data_or_file_object.name)

    requirements_data = yaml.safe_load(data_or_file_object)

    # log.debug('requirements_data: %s', pprint.pformat(requirements_data))

    requirements_list = []

    for req_data_item in requirements_data:
        # log.debug('req_data_item: %s', req_data_item)
        # log.debug('type(req_data_item): %s', type(req_data_item))

        req_spec_data = yaml_parse.yaml_parse(req_data_item)

        # FIXME: find a better place to do this
        # create a version_spec from the 'version' field, a requiremenst.yml
        # does not currently expect to have a 'version_spec' data in it
        version_spec_str = '*'
        version_str = req_spec_data.pop('version')

        if version_str:
            version_spec_str = '==%s' % version_str

        version_spec = semantic_version.Spec(version_spec_str)

        req_spec_data['version_spec'] = version_spec

        # log.debug('data_name (after): %s', data_name)
        # log.debug('name_info: %s', name_info)

        req_spec = RequirementSpec.from_dict(req_spec_data)

        # log.debug('req_spec: %s', req_spec)

        req = Requirement(repository_spec=repository_spec, op=RequirementOps.EQ, requirement_spec=req_spec)

        # log.debug('req: %s', req)

        requirements_list.append(req)

    log.debug('FINISH of load of requirements: %s: %s', data_or_file_object.name, requirements_list)
    return requirements_list


def from_dependencies_dict(dependencies_dict, namespace_override=None, editable=False, repository_spec=None):
    '''Build a list of Requirement objects from the 'dependencies' item in galaxy.yml'''
    reqs = []
    for req_label, req_version_spec in dependencies_dict.items():
        req_spec_data = spec_data_from_string(req_label,
                                              namespace_override=namespace_override,
                                              editable=editable)
        req_spec_data['version_spec'] = req_version_spec
        req_spec_data['req_spec_string'] = req_spec_data.pop('spec_string', None)

        log.debug('req_spec_data: %s', req_spec_data)

        req_spec = RequirementSpec.from_dict(req_spec_data)

        log.debug('req_spec: %s', req_spec)

        requirement = Requirement(repository_spec=repository_spec, op=RequirementOps.EQ,
                                  scope=RequirementScopes.INSTALL,
                                  requirement_spec=req_spec)
        log.debug('requirement: %s', requirement)
        reqs.append(requirement)

    return reqs


def from_requirement_spec_strings(requirement_spec_strings, namespace_override=None, editable=False, repository_spec=None):
    reqs = []
    for requirement_spec_string in requirement_spec_strings:
        req_spec_data = spec_data_from_string(requirement_spec_string,
                                              namespace_override=namespace_override,
                                              editable=editable)

        req_spec = RequirementSpec.from_dict(req_spec_data)

        req = Requirement(repository_spec=repository_spec, op=RequirementOps.EQ, requirement_spec=req_spec)

        reqs.append(req)

    return reqs



import attr
import logging
import os
import pytest
import yaml

from ansible_galaxy import collection_info
from ansible_galaxy.models.collection_info import CollectionInfo
from ansible_galaxy import yaml_persist
from ansible_galaxy import exceptions

log = logging.getLogger(__name__)


def test_load():
    file_name = "example_collection_info1.yml"
    test_data_path = os.path.join(os.path.dirname(__file__), '%s' % file_name)
    expected = {'namespace': 'some_namespace',
                'name': 'some_name',
                'version': '11.11.11',
                'authors': ['Carlos Boozer'],
                'description': 'something',
                'license': ['GPL-3.0-or-later'],
                'license_file': None,
                'tags': [],
                'readme': None,
                'documentation': None,
                'homepage': None,
                'issues': None,
                'repository': None,
                'dependencies': {}}

    with open(test_data_path, 'r') as data_fd:
        res = collection_info.load(data_fd)
        log.debug('res: %s', res)

        assert isinstance(res, CollectionInfo)
        res_dict = attr.asdict(res)
        assert res_dict == expected

        for key in res_dict:
            assert getattr(res, key) == res_dict[key] == expected[key]
            assert isinstance(getattr(res, key), type(expected[key]))


def test_save(tmpdir):
    temp_dir = tmpdir.mkdir('mazer_collectio_info_unit_test')

    file_name = "example_collection_info1.yml"

    temp_file = temp_dir.join(file_name)

    test_data_path = os.path.join(os.path.dirname(__file__), '%s' % file_name)
    with open(test_data_path, 'r') as data_fd:
        res = collection_info.load(data_fd)

        log.debug('temp_file.strpath: %s', temp_file.strpath)
        yaml_persist.save(res, temp_file.strpath)

        # open the save()'ed file and load a new collection_artifact_manifest from it
        with open(temp_file.strpath, 'r') as read_fd:
            reloaded = collection_info.load(read_fd)

            log.debug('reloaded: %s', reloaded)

            # verify the object loaded from known example matches the example after
            # a save()/load() cycle
            assert reloaded == res

            # read the file again and log the file contents
            read_fd.seek(0)
            buf = read_fd.read()
            log.debug('buf: %s', buf)


def test_parse_error(tmpdir):
    test_data = {
        'name': 'foo.foo',
        'authors': ['chouseknecht'],
        'license': 'GPL-3.0-or-later',
        'version': '0.0.1',
        'description': 'unit testing thing',
        'foo': 'foo',
    }
    collection_yaml = yaml.safe_dump(test_data, stream=None)
    with pytest.raises(exceptions.GalaxyClientError):
        collection_info.load(collection_yaml)



from collections import OrderedDict
import logging
import os

import attr
import yaml

from ansible_galaxy.utils import attr_utils

log = logging.getLogger(__name__)


class MazerSafeDumper(yaml.SafeDumper):
    '''A Yaml dumper that dumps more or less ansible-style

    ie, two space indent on lists always, flow_style=False
    on mappings

    Also adds support for OrderedDict'''
    def increase_indent(self, flow=False, indentless=False):
        return super(MazerSafeDumper, self).increase_indent(flow, False)

    def represent_data(self, data):
        '''Attempt to detect attrs style objects and convert to dicts

        We can't add a regular yaml_representer because attrs based objects can
        be of any type, so we need to detect if 'data' is attrs based (with
        attr_utils.is_attrs()) and if so, we covert it to an OrdedDict and
        then rely on the OrderedDict representer to dump the correct yaml.

        If this is anything other than an attrs based instance, we call
        super's represent_data()

        Also setup some representers with the default styles are 'ansible-style'
        for maps/dicts/lists/tuples
        '''

        if attr_utils.is_attr(data):
            odict = attr.asdict(data, recurse=True, dict_factory=OrderedDict)
            return super(MazerSafeDumper, self).represent_data(odict)

        return super(MazerSafeDumper, self).represent_data(data)


def dict_representer(dumper, data):
    return dumper.represent_mapping(u'tag:yaml.org,2002:map', data, flow_style=False)


def odict_representer(dumper, data):
    return dumper.represent_mapping(u'tag:yaml.org,2002:map', data.items(), flow_style=False)


def list_representer(dumper, data):
    return dumper.represent_sequence(u'tag:yaml.org,2002:seq', data, flow_style=False)


MazerSafeDumper.add_representer(OrderedDict, odict_representer)
MazerSafeDumper.add_representer(dict, dict_representer)
MazerSafeDumper.add_representer(list, list_representer)
MazerSafeDumper.add_representer(tuple, list_representer)


def safe_dump(data, stream=None, **kwargs):
    return yaml.dump(data, stream=stream, Dumper=MazerSafeDumper, **kwargs)


def load_from_filename(filename, load_method):
    if not os.path.isfile(filename):
        return None

    try:
        f = open(filename, 'r')
        return load_method(f)
    except Exception as e:
        log.exception(e)
        log.debug('Unable to load install info from path: %s', filename)
        return False
    finally:
        f.close()


def save(data, filename):
    log.debug('saving data to %s', filename)
    if not os.path.exists(os.path.dirname(filename)):
        os.makedirs(os.path.dirname(filename))

    with open(filename, 'w+') as f:
        # FIXME: just return the install_info dict (or better, build it elsewhere and pass in)
        # FIXME: stop minging self state
        try:
            install_info_ = safe_dump(data, stream=f)
        except Exception as e:
            log.warning('unable to serialize data to filename=%s for data=%s', filename, install_info_)
            log.exception(e)
            return False

    log.debug('wrote data to %s', filename)
    return True




import logging

from ansible_galaxy.models import content

log = logging.getLogger(__name__)


def test_content_type_dir_map():
    assert isinstance(content.CONTENT_TYPE_DIR_MAP, dict)


def test_content_type_dir_map_items():
    for content_type in content.CONTENT_TYPES:
        assert content_type in content.CONTENT_TYPE_DIR_MAP
        expected = '%ss' % content_type
        assert content.CONTENT_TYPE_DIR_MAP[content_type] == expected

    assert 'role' in content.CONTENT_TYPES



import logging
import os

import yaml

from ansible_galaxy import dependencies
from ansible_galaxy.models.role_metadata import RoleMetadata


log = logging.getLogger(__name__)


def load(data_or_file_object, role_name=None):
    # log.debug('loading role metadata from %s', data_or_file_object)

    # TODO: potentially want to let yaml errors bubble up more
    #       so errors have more useful info if there is yaml parse error
    data_dict = yaml.safe_load(data_or_file_object)

    # log.debug('data_dict: %s', pprint.pformat(data_dict))

    galaxy_info = data_dict.get('galaxy_info', {})

    dependencies_data = data_dict.get('dependencies', [])

    deps = dependencies.from_dependency_spec_strings(dependencies_data)

    allow_duplicates = data_dict.get('allow_duplicates', False)

    # we pass in the dir name of the role as role_name by default, but
    # can override with role_name in main.yml
    if galaxy_info.get('role_name', None):
        role_name = role_name

    role_md = RoleMetadata(name=role_name,
                           author=galaxy_info.get('author', None),
                           description=galaxy_info.get('description', None),
                           company=galaxy_info.get('company', None),
                           license=galaxy_info.get('license', None),
                           galaxy_tags=galaxy_info.get('galaxy_tags'),
                           platforms=galaxy_info.get('platforms'),
                           cloud_platforms=galaxy_info.get('cloud_platforms'),
                           min_ansible_version=galaxy_info.get('min_ansible_version'),
                           min_ansible_container_version=galaxy_info.get('min_ansible_container_version'),
                           issue_tracker=galaxy_info.get('issue_tracker_url'),
                           # from outside of galaxy_info dict
                           dependencies=deps,
                           allow_duplicates=allow_duplicates)

    # log.debug('loaded role metadata: %s', role_md)
    log.debug('loaded role metadata for %s from %s', role_name, data_or_file_object)
    return role_md


def load_from_filename(filename, role_name=None):
    log.debug('looking for content meta data from path: %s', filename)

    if not os.path.isfile(filename):
        return None

    try:
        f = open(filename, 'r')
        return load(f, role_name=role_name)
    except Exception as e:
        log.exception(e)
        log.debug('Unable to load role metadata from path: %s', filename)
        return False
    finally:
        f.close()


def load_from_dir(dirname, role_name=None):
    log.debug("Going to load role from dir %s %s", dirname, role_name or "")

    role_meta_main_filename = os.path.join(dirname, 'meta', 'main.yml')
    # role_name = '%s.%s' % (namespace, name)

    return load_from_filename(role_meta_main_filename, role_name=role_name)



from collections import OrderedDict
import logging
import tempfile

from ansible_galaxy.config import config

log = logging.getLogger(__name__)

CONFIG_SECTIONS = ['server', 'content_path', 'global_content_path', 'options']


def assert_object(config_obj):
    assert isinstance(config_obj, config.Config)

    for attr in CONFIG_SECTIONS:
        assert hasattr(config_obj, attr), 'Config instance does not have attr "%s"' % attr


def test_config_init():
    config_ = config.Config()

    assert_object(config_)


def test_config_unknown_attr():
    config_ = config.Config()

    assert hasattr(config_, 'not_a_valid_attr') is False
    try:
        blip = config_.not_a_valid_attr
    except AttributeError as e:
        log.debug(e, exc_info=True)
        return

    assert False, 'Expected to get an AttributeError accessing an unknown attr but did not and %s was returned' % blip


def test_config_from_empty_dict():
    config_data = OrderedDict({})
    config_ = config.Config.from_dict(config_data)
    assert_object(config_)


def test_config_from_dict():
    config_data = OrderedDict([
        ('options', {'some_option': 'some_option_value'}),
    ])

    config_ = config.Config.from_dict(config_data)
    assert_object(config_)

    assert config_.options['some_option'] == 'some_option_value'


def test_config_as_dict_empty():
    config_ = config.Config()
    config_data = config_.as_dict()

    assert isinstance(config_data, dict)

    assert set(config_data) == set(CONFIG_SECTIONS)


def test_config_as_dict():
    orig_config_data = OrderedDict([
        ('server', {'url': 'some_url_value',
                    'ignore_certs': True}),
        ('content_path', None),
        ('global_content_path', None),
        ('options', {'some_option': 'some_option_value'}),
    ])

    config_ = config.Config.from_dict(orig_config_data)
    config_data = config_.as_dict()

    assert isinstance(config_data, dict)

    # verify we end up with keys in dict corresponding to each config section
    assert set(config_data) == set(CONFIG_SECTIONS)

    assert config_data == orig_config_data


def test_config_as_dict_from_partial_dict():
    orig_config_data = {
        'server': {'url': 'some_url_value'},
    }
    config_ = config.Config.from_dict(orig_config_data)
    config_data = config_.as_dict()

    assert isinstance(config_data, dict)

    assert set(config_data) == set(CONFIG_SECTIONS)

    assert config_data['server'] == orig_config_data['server']

    assert isinstance(config_data['options'], dict)
    assert config_data['options'] == {}

    assert config_data['content_path'] is None


def test_load_empty():
    yaml_fo = tempfile.NamedTemporaryFile()

    config_ = config.load(yaml_fo.name)

    assert_object(config_)
    log.debug('data: %s', config_.as_dict())


def test_save_empty():
    yaml_fo = tempfile.NamedTemporaryFile()

    config_ = config.Config()

    config.save(config_, yaml_fo.name)

    res_fo = open(yaml_fo.name, 'r')

    written_yaml = res_fo.read()
    log.debug('written_yaml: %s', written_yaml)

    assert written_yaml != ''
    assert 'server' in written_yaml
    assert 'content_path' in written_yaml


def test_save():
    yaml_fo = tempfile.NamedTemporaryFile()

    config_ = config.Config()

    config_.options['some_option'] = 'some_option_value'

    config.save(config_, yaml_fo.name)

    res_fo = open(yaml_fo.name, 'r')

    written_yaml = res_fo.read()
    log.debug('written_yaml: %s', written_yaml)

    assert written_yaml != ''
    expected_strs = [
        'some_option',
        'some_option_value',
    ]
    for expected_str in expected_strs:
        assert expected_str in written_yaml, \
            'expected to find the string "%s" in written config file but did not. file contents: %s' % (expected_str, written_yaml)



import logging
import six

from ansible_galaxy import repository_spec_parse
from ansible_galaxy.utils.role_spec import role_spec_parse
from ansible_galaxy.models.content import VALID_ROLE_SPEC_KEYS


log = logging.getLogger(__name__)


# FIXME: not really yaml,
# FIXME: whats the diff between this and role_spec_parse?
# TODO: dont munge the passed in content
# TODO: split into smaller methods
# FIXME: does this actually use yaml?
# FIXME: kind of seems like this does two different things
# FIXME: letting this take a string _or_ a mapping seems troubleprone
def yaml_parse(content, resolver=None):
    """parses the passed in yaml string and returns a dict with name/src/scm/version

    Or... if the passed in 'content' is a dict, it either creates role or if not a role,
    it copies the dict and sets name/src/scm/version in it"""

    # FIXME: rm once we stop clobbering the passed in content, we can stop making a
    #        copy of orig_content as this is just for logging
    #        the original value

    if isinstance(content, six.string_types):
        log.debug('parsing content="%s" as a string', content)

        # orig_content = copy.deepcopy(content)
        res = repository_spec_parse.spec_data_from_string(content, resolver=resolver)

        log.debug('parsed spec="%s" -> %s', content, res)

        return res

    # log.debug('content="%s" is not a string (it is a %s) so we are assuming it is a dict',
    #          content, type(content))

    # orig_content = copy.deepcopy(content)

    # Not sure what will/should happen if content is not a Mapping or a string
    # FIXME: if content is not a string or a dict/map, throw a reasonable error.
    #        for ex, if a list of strings is passed in, the content.copy() below throws
    #        an attribute error
    # FIXME: This isn't a 'parse' at all, if we want to convert a dict/map to the specific
    #        type of dict we expect, should be a different method
    # FIXME: what is expected to happen if passed an empty dict?
    if 'role' in content:
        log.debug('content="%s" appears to be a role', content)

        name = content['role']
        if ',' in name:
            # Old style: {role: "galaxy.role,version,name", other_vars: "here" }
            # Maintained for backwards compat
            content = role_spec_parse(content['role'])
        else:
            del content['role']
            content['name'] = name
    else:
        # log.debug('content="%s" does not appear to be a role', content)

        # FIXME: just use a new name already
        # FIXME: this fails for objects with no dict attribute, like a list
        content_copy = content.copy()

        # log.debug('content_copy: %s', content_copy)

        data = {'src': None, 'version': None, 'name': None, 'scm': None}
        data.update(content_copy)
        content = data

        if data.get('src', None):
            # valid_kw = ('src', 'version', 'name', 'scm')
            new_data = repository_spec_parse.spec_data_from_string(data['src'])

            # log.debug('new_data: %s', new_data)

            for key in new_data:
                if not data.get(key, None):
                    data[key] = new_data[key]

            # New style: { src: 'galaxy.role,version,name', other_vars: "here" }
            if 'github.com' in content["src"] and 'http' in content["src"] and '+' not in content["src"] and not content["src"].endswith('.tar.gz'):
                content["src"] = "git+" + content["src"]

        if 'version' not in content:
            content['version'] = ''

        if 'scm' not in content:
            content['scm'] = None

    for key in list(content.keys()):
        # sub_name doesnt mean anything to role spec
        if key not in VALID_ROLE_SPEC_KEYS and key not in ('sub_name', 'namespace', 'fetch_method'):
            # log.debug('removing invalid key: %s', key)

            content.pop(key)

    # log.debug('"parsed" content="%s" into: %s', orig_content, content)

    return content



import logging
import pprint

from ansible_galaxy import collection_artifact
from ansible_galaxy import display
from ansible_galaxy import exceptions
from ansible_galaxy import install
from ansible_galaxy import installed_repository_db
from ansible_galaxy import matchers
# from ansible_galaxy import requirements
from ansible_galaxy import repository_spec
from ansible_galaxy.fetch import fetch_factory
from ansible_galaxy.models.repository_spec import FetchMethods
from ansible_galaxy.models.requirement import Requirement, RequirementOps
from ansible_galaxy.models.requirement_spec import RequirementSpec

log = logging.getLogger(__name__)


def raise_without_ignore(ignore_errors, msg=None, rc=1):
    """
    Exits with the specified return code unless the
    option --ignore-errors was specified
    """
    ignore_error_blurb = '- you can use --ignore-errors to skip failed roles and finish processing the list.'
    if not ignore_errors:
        # Note: msg may actually be an exception instance or a text string

        message = ignore_error_blurb
        if msg:
            message = '%s:\n%s' % (msg, ignore_error_blurb)
        # TODO: some sort of ignoreable exception
        raise exceptions.GalaxyError(message)


def _verify_requirements_repository_spec_have_namespaces(requirements_list):
    for requirement_to_install in requirements_list:
        req_spec = requirement_to_install.requirement_spec
        log.debug('repo install repository_spec: %s', req_spec)

        if not req_spec.namespace:
            raise exceptions.GalaxyRepositorySpecError(
                'The repository spec "%s" requires a namespace (either "namespace.name" or via --namespace)' % req_spec.spec_string,
                repository_spec=req_spec)


# pass a list of repository_spec objects
def install_repositories_matching_repository_specs(galaxy_context,
                                                   requirements_list,
                                                   editable=False,
                                                   namespace_override=None,
                                                   display_callback=None,
                                                   # TODO: error handling callback ?
                                                   ignore_errors=False,
                                                   no_deps=False,
                                                   force_overwrite=False):
    '''Install a set of repositories specified by repository_specs if they are not already installed'''

    # log.debug('editable: %s', editable)
    log.debug('requirements_list: %s', requirements_list)

    _verify_requirements_repository_spec_have_namespaces(requirements_list)

    # FIXME: mv mv this filtering to it's own method
    # match any of the content specs for stuff we want to install
    # ie, see if it is already installed
    requested_repository_specs = [x.requirement_spec for x in requirements_list]
    repository_spec_match_filter = matchers.MatchRepositorySpecNamespaceName(requested_repository_specs)

    irdb = installed_repository_db.InstalledRepositoryDatabase(galaxy_context)
    already_installed_generator = irdb.select(repository_spec_match_filter=repository_spec_match_filter)

    # FIXME: if/when GalaxyContent and InstalledGalaxyContent are attr.ib based and frozen and hashable
    #        we can simplify this filter with set ops

    already_installed_repository_spec_set = set([installed.repository_spec for installed in already_installed_generator])
    log.debug('already_installed_repository_spec_set: %s', already_installed_repository_spec_set)

    # This filters out already installed repositories unless --force. Aside from the warning, 'mazer install alikins.something_installed_already' is ok.
    requirements_to_install = [y for y in requirements_list if y.requirement_spec not in already_installed_repository_spec_set and not force_overwrite]

    log.debug('repository_specs_to_install: %s', pprint.pformat(requirements_to_install))

    return install_repositories(galaxy_context, requirements_to_install,
                                display_callback=display_callback,
                                ignore_errors=ignore_errors,
                                no_deps=no_deps,
                                force_overwrite=force_overwrite)


# FIXME: probably pass the point where passing around all the data to methods makes sense
#        so probably needs a stateful class here
def install_repository_specs_loop(galaxy_context,
                                  repository_spec_strings=None,
                                  requirements_list=None,
                                  editable=False,
                                  namespace_override=None,
                                  display_callback=None,
                                  # TODO: error handling callback ?
                                  ignore_errors=False,
                                  no_deps=False,
                                  force_overwrite=False):

    requirements_list = requirements_list or []

    for repository_spec_string in repository_spec_strings:
        fetch_method = \
            repository_spec.choose_repository_fetch_method(repository_spec_string,
                                                           editable=editable)
        log.debug('fetch_method: %s', fetch_method)

        if fetch_method == FetchMethods.LOCAL_FILE:
            # Since only know this is a local file we vaguely recognize, we have to
            # open it up to get any more details. We _could_ attempt to parse the file
            # name, but that rarely ends well...
            spec_data = collection_artifact.load_data_from_collection_artifact(repository_spec_string)
            spec_data['fetch_method'] = fetch_method
        else:
            spec_data = repository_spec.spec_data_from_string(repository_spec_string,
                                                              namespace_override=namespace_override,
                                                              editable=editable)

            spec_data['fetch_method'] = fetch_method

        log.debug('spec_data: %s', spec_data)

        req_spec = RequirementSpec.from_dict(spec_data)

        req = Requirement(repository_spec=None, op=RequirementOps.EQ, requirement_spec=req_spec)

        requirements_list.append(req)

    log.debug('requirements_list: %s', requirements_list)
    for req in requirements_list:
        display_callback('Installing %s' % req.requirement_spec.label, level='info')

    while True:
        if not requirements_list:
            break

        just_installed_repositories = \
            install_repositories_matching_repository_specs(galaxy_context,
                                                           requirements_list,
                                                           editable=editable,
                                                           namespace_override=namespace_override,
                                                           display_callback=display_callback,
                                                           ignore_errors=ignore_errors,
                                                           no_deps=no_deps,
                                                           force_overwrite=force_overwrite)

        # set the repository_specs to search for to whatever the install reported as being needed yet
        # requirements_list = new_requirements_list
        requirements_list = find_new_deps_from_installed(galaxy_context,
                                                         just_installed_repositories,
                                                         no_deps=no_deps)

        for req in requirements_list:
            if req.repository_spec:
                msg = 'Installing requirement %s (required by %s)' % (req.requirement_spec.label, req.repository_spec.label)
            else:
                msg = 'Installing requirement %s' % req.requirement_spec.label
            display_callback(msg, level='info')

    # FIXME: what results to return?
    return 0


def find_new_deps_from_installed(galaxy_context, installed_repos, no_deps=False):
    if no_deps:
        return []

    # FIXME: Just return the single item list installed_repositories here
    deps_and_reqs_set = set()

    log.debug('finding new deps for installed repos: %s',
              [str(x) for x in installed_repos])

    # Remove dupes. Note, can/will change ordering.
    # installed_repos = list(set(installed_repos))

    # install dependencies, if we want them
    for installed_repository in installed_repos:
        # log.debug('just_installed_repository: %s', installed_repository)

        # convert deps/reqs to sets. Losing any ordering, but avoids dupes
        reqs_set = set(installed_repository.requirements)

        deps_set = set(installed_repository.dependencies)

        deps_and_reqs_set.update(deps_set, reqs_set)

        # for dep_req in sorted(deps_and_reqs_set):
        #    log.debug('deps_and_reqs_set_item: %s', dep_req)

    deps_and_reqs_list = sorted(list(deps_and_reqs_set))

    # log.debug('deps_and_reqs_list: %s', pf(deps_and_reqs_list))

    unsolved_deps_reqs = []
    for dep_req in deps_and_reqs_list:
        log.debug('Checking if %s is provided by something installed', str(dep_req))

        # Search for an exact ns_n_v match
        irdb = installed_repository_db.InstalledRepositoryDatabase(galaxy_context)
        already_installed_iter = irdb.by_requirement(dep_req)
        already_installed = list(already_installed_iter)

        log.debug('already_installed: %s', already_installed)

        solved = False
        for provider in already_installed:
            log.debug('The dep_req %s is already provided by %s', dep_req, provider)
            solved = solved or True

        if solved:
            log.debug('skipping dep_req %s', dep_req)
            continue

        unsolved_deps_reqs.append(dep_req)

    log.debug('Found additional requirements: %s', pprint.pformat(unsolved_deps_reqs))

    return unsolved_deps_reqs


# TODO: split into resolve, find/get metadata, resolve deps, download, install transaction
def install_repositories(galaxy_context,
                         requirements_to_install,
                         display_callback=None,
                         # TODO: error handling callback ?
                         ignore_errors=False,
                         no_deps=False,
                         force_overwrite=False):

    display_callback = display_callback or display.display_callback
    log.debug('requirements_to_install: %s', requirements_to_install)
    # log.debug('no_deps: %s', no_deps)
    # log.debug('force_overwrite: %s', force_overwrite)

    # dep_requirements = []
    most_installed_repositories = []

    # FIXME - Need to handle role files here for backwards compat

    # TODO: this should be adding the content/self.args/content_left to
    #       a list of needed deps

    # Remove any dupe repository_specs
    requirements_to_install_uniq = set(requirements_to_install)

    # TODO: if the default ordering of repository_specs isnt useful, may need to tweak it
    for requirement_to_install in sorted(requirements_to_install_uniq):
        log.debug('requirement_to_install: %s', requirement_to_install)
        installed_repositories = install_repository(galaxy_context,
                                                    requirement_to_install,
                                                    display_callback=display_callback,
                                                    ignore_errors=ignore_errors,
                                                    no_deps=no_deps,
                                                    force_overwrite=force_overwrite)

        # log.debug('dep_requirement_repository_specs1: %s', dep_requirements)

        if not installed_repositories:
            log.debug('install_repository() returned None for requirement_to_install: %s', requirement_to_install)
            continue

        for installed_repo in installed_repositories:
            required_by_blurb = ''
            if requirement_to_install.repository_spec:
                required_by_blurb = ' (required by %s)' % requirement_to_install.repository_spec.label

            log.info('Installed: %s %s to %s%s',
                     installed_repo.label,
                     installed_repo.repository_spec.version,
                     installed_repo.path,
                     required_by_blurb)

        most_installed_repositories.extend(installed_repositories)
        # dep_requirements.extend(new_dep_requirements)

    return most_installed_repositories


def install_repository(galaxy_context,
                       requirement_to_install,
                       display_callback=None,
                       # TODO: error handling callback ?
                       ignore_errors=False,
                       no_deps=False,
                       force_overwrite=False):
    '''This installs a single package by finding it, fetching it, verifying it and installing it.'''

    display_callback = display_callback or display.display_callback

    # INITIAL state
    # dep_requirements = []

    # TODO: we could do all the downloads first, then install them. Likely
    #       less error prone mid 'transaction'
    log.debug('Processing %r', requirement_to_install)

    repository_spec_to_install = requirement_to_install.requirement_spec
    requirement_spec_to_install = requirement_to_install.requirement_spec

    # else trans to ... FIND_FETCHER?

    # TODO: check if already installed and move to approriate state

    log.debug('About to find() requested requirement_spec_to_install: %s', requirement_spec_to_install)

    # potential_repository_spec is a repo spec for the install candidate we potentially found.
    irdb = installed_repository_db.InstalledRepositoryDatabase(galaxy_context)
    log.debug('Checking to see if %s is already installed', requirement_spec_to_install)

    already_installed_iter = irdb.by_requirement_spec(requirement_spec_to_install)
    already_installed = sorted(list(already_installed_iter))

    log.debug('already_installed: %s', already_installed)

    if already_installed:
        for already_installed_repository in already_installed:
            display_callback('%s is already installed at %s' % (already_installed_repository.repository_spec.label,
                                                                already_installed_repository.path),
                             level='warning')
        log.debug('Stuff %s was already installed. In %s', requirement_spec_to_install, already_installed)

        return None

    # We dont have anything that matches the RequirementSpec installed
    fetcher = fetch_factory.get(galaxy_context=galaxy_context,
                                requirement_spec=requirement_spec_to_install)

    # if we fail to get a fetcher here, then to... FIND_FETCHER_FAILURE ?
    # could also move some of the logic in fetcher_factory to be driven from here
    # and make the steps of mapping repository spec -> fetcher method part of the
    # state machine. That might be a good place to support multiple galaxy servers
    # or preferring local content to remote content, etc.

    # FIND state
    # See if we can find metadata and/or download the archive before we try to
    # remove an installed version...
    try:
        find_results = install.find(fetcher)
    except exceptions.GalaxyError as e:
        log.warning('Unable to find metadata for %s: %s', requirement_spec_to_install.label, e)
        # FIXME: raise dep error exception?
        raise_without_ignore(ignore_errors, e)
        # continue
        return None

    # TODO: make sure repository_spec version is correct and set

    # TODO: state transition, if find_results -> INSTALL
    #       if not, then FIND_FAILED

    # TODO/FIXME: We give find() a RequirementSpec, but find_results should have enough
    #             info to create a concrete RepositorySpec

    # TODO: if we want client side content whitelist/blacklist, or pinned versions,
    #       or rules to only update within some semver range (ie, only 'patch' level),
    #       we could hook rule validation stuff here.

    # TODO: build a new repository_spec based on what we actually fetched to feed to
    #       install etc. The fetcher.fetch() could return a datastructure needed to build
    #       the new one instead of doing it in verify()
    found_repository_spec = install.repository_spec_from_find_results(find_results,
                                                                      requirement_spec_to_install)

    log.debug('found_repository_spec: %s', found_repository_spec)

    repository_spec_to_install = found_repository_spec
    log.debug('About to download repository requested by %s: %s', requirement_spec_to_install, repository_spec_to_install)

    # FETCH state
    try:
        fetch_results = install.fetch(fetcher,
                                      repository_spec=repository_spec_to_install,
                                      find_results=find_results)
        log.debug('fetch_results: %s', fetch_results)
        # fetch_results will include a 'archive_path' pointing to where the artifact
        # was saved to locally.
    except exceptions.GalaxyError as e:
        # fetch error probably should just go to a FAILED state, at least until
        # we have to implement retries
        log.warning('Unable to fetch %s: %s', repository_spec_to_install.name, e)
        raise_without_ignore(ignore_errors, e)
        # continue
        # FIXME: raise ?
        return None

    # FIXME: seems like we want to resolve deps before trying install
    #        We need the role (or other content) deps from meta before installing
    #        though, and sometimes (for galaxy case) we dont know that until we've downloaded
    #        the file, which we dont do until somewhere in the begin of content.install (fetch).
    #        We can get that from the galaxy API though.
    #
    # FIXME: exc handling

    installed_repositories = []

    try:
        installed_repositories = install.install(galaxy_context,
                                                 fetcher,
                                                 fetch_results,
                                                 repository_spec=found_repository_spec,
                                                 force_overwrite=force_overwrite,
                                                 display_callback=display_callback)
    except exceptions.GalaxyError as e:
        msg = "- %s was NOT installed successfully: %s "
        display_callback(msg % (found_repository_spec.label, e), level='warning')
        log.warning(msg, found_repository_spec.label, str(e))
        raise_without_ignore(ignore_errors, e)
        return []

    if not installed_repositories:
        log.warning("- %s was NOT installed successfully.", found_repository_spec.label)
        raise_without_ignore(ignore_errors)

    return installed_repositories




import logging

import six

from ansible_galaxy.models import context

log = logging.getLogger(__name__)


def test_context_empty_init():
    galaxy_context = context.GalaxyContext()

    assert galaxy_context.server is not None
    assert galaxy_context.content_path is None
    assert isinstance(galaxy_context.server, dict)


def test_context_with_content_path_and_server():
    content_path = '/dev/null/some_content_path'
    server_url = 'http://example.com:9999/'
    ignore_certs = False

    server = {'url': server_url,
              'ignore_certs': ignore_certs}

    galaxy_context = context.GalaxyContext(server=server, content_path=content_path)

    log.debug('galaxy_context: %s', galaxy_context)
    assert isinstance(galaxy_context, context.GalaxyContext)

    assert isinstance(galaxy_context.content_path, six.string_types)
    assert isinstance(galaxy_context.server, dict)

    assert galaxy_context.server['url'] == server_url
    assert galaxy_context.server['ignore_certs'] == ignore_certs

    assert galaxy_context.content_path == content_path


def test_context_from_empty_server():
    server = {}
    galaxy_context = context.GalaxyContext(server=server)

    assert galaxy_context.content_path is None
    assert isinstance(galaxy_context.server, dict)
    log.debug('server: %s', galaxy_context.server)
    assert galaxy_context.server['url'] is None
    assert galaxy_context.server['ignore_certs'] is False


def test_context_server_none_content_path_none():

    galaxy_context = context.GalaxyContext(server=None,
                                           content_path=None)

    assert galaxy_context.content_path is None
    assert isinstance(galaxy_context.server, dict)
    assert galaxy_context.server['url'] is None
    assert galaxy_context.server['ignore_certs'] is False


def test_context_repr():
    content_path = '/dev/null/some_content_path'
    server_url = 'http://example.com:9999/'
    ignore_certs = False

    server = {'url': server_url,
              'ignore_certs': ignore_certs}

    galaxy_context = context.GalaxyContext(server=server, content_path=content_path)
    rep_res = repr(galaxy_context)

    log.debug('rep_res: %s', rep_res)

    assert isinstance(rep_res, six.string_types)
    assert 'content_path' in rep_res
    assert 'server' in rep_res
    assert 'some_content_path' in rep_res




import logging

import pytest
import semantic_version

from ansible_galaxy import repository_version
from ansible_galaxy import exceptions
from ansible_galaxy.models.requirement_spec import RequirementSpec

log = logging.getLogger(__name__)


any_version = semantic_version.Spec('*')


def test_get_content_version_none():
    req_spec_data = {'namespace': 'some_namespace',
                     'name': 'some_name',
                     'version': None}
    req_spec = RequirementSpec.from_dict(req_spec_data)
    with pytest.raises(exceptions.GalaxyError, match="The list of available versions for some_namespace.some_name is empty"):
        repository_version.get_repository_version({}, req_spec, [])


@pytest.mark.xfail
def test_get_content_version_prod_version_in_content_versions():
    ret = repository_version.get_repository_version({}, 'prod', content_versions_147, None)
    assert ret == 'prod'


raw_content_versions_147 = [
    {
        "id": 126,
        "name": "2.3.7",
        "release_date": "2018-05-01T12:10:16-04:00",
        "url": "",
        "related": {},
        "summary_fields": {},
        "created": "2018-05-01T16:11:41.473282Z",
        "modified": "2018-05-01T16:11:42.110659Z",
        "active": True
    },
    {
        "id": 127,
        "name": "2.3.6",
        "release_date": "2018-05-01T11:54:19-04:00",
        "url": "",
        "related": {},
        "summary_fields": {},
        "created": "2018-05-01T16:11:42.114698Z",
        "modified": "2018-05-01T16:11:42.427811Z",
        "active": True
    },
]

content_versions_147 = [a.get('name') for a in raw_content_versions_147 if a.get('name', None)]
content_versions_v = content_versions_147 + ['v1.0.0', 'v3.0.0', 'v0.0.0']
content_versions_no_v = content_versions_147 + ['1.0.0', '3.0.0', '0.0.0']
content_versions_1_0_v_and_no_v = ['1.0.0', 'v1.0.0']
content_versions_v_and_no_v = content_versions_147 + content_versions_1_0_v_and_no_v

content_versions_boring = ['0.0.1', '0.0.11', '0.5.1', '0.99.0', '1.0.0', '2.0.0', '3.0.0']
content_versions_alphanumeric_post = ['0.0.1beta', '0.0.2a']
versions_1_0_0_latest = ['0.0.0', '0.1', '0.5.5.5.5.5', '1.0.0']

content_versions_map = {
    '147': content_versions_147,
    'v': content_versions_v,
    'no_v': content_versions_no_v,
    '1_0_v_and_no_v': content_versions_1_0_v_and_no_v,
    'v_and_no_v': content_versions_v_and_no_v,
    'boring': content_versions_boring,
    '1_0_0_latest': versions_1_0_0_latest,
    'empty': [],
}

test_data = [
    {'ask': 'v1.0.0', 'vid': 'v', 'exp': 'v1.0.0'},
    {'ask': 'v1.0.0', 'vid': 'no_v', 'exp': '1.0.0'},
    {'ask': '1.0.0', 'vid': 'v', 'exp': 'v1.0.0'},
    {'ask': '1.0.0', 'vid': 'no_v', 'exp': '1.0.0'},
    {'ask': '2.3.7', 'vid': '147', 'exp': '2.3.7'},
    {'ask': 'v2.3.7', 'vid': '147', 'exp': '2.3.7'},
]


@pytest.fixture(scope='module',
                params=test_data,
                ids=['ask=%s,vlist=%s,exp=%s' % (x['ask'], x['vid'], x['exp']) for x in test_data])
def ver_data(request):
    tdata = request.param.copy()
    tdata['vlist'] = content_versions_map[tdata['vid']]
    log.debug('tdata: %s', tdata)
    yield tdata


test_sort_data = [
    {'vid': 'empty', 'latest': None},
    {'vid': 'no_v', 'latest': '3.0.0'},
    {'vid': 'boring', 'latest': '3.0.0'},
    {'vid': '147', 'latest': '2.3.7'},
]


@pytest.fixture(scope='module',
                params=test_sort_data,
                ids=['vlist=%s,latest=%s' % (x['vid'], x['latest']) for x in test_sort_data])
def ver_sort_data(request):
    tdata = request.param.copy()
    tdata['vlist'] = content_versions_map[tdata['vid']]
    log.debug('tdata: %s', tdata)
    yield tdata


def test_sort_versions(ver_sort_data):
    log.debug('ver_data_sort: %s', ver_sort_data)
    log.debug('latest: %s', ver_sort_data['latest'])
    versions = ver_sort_data['vlist']

    res = repository_version.sort_versions(versions)
    log.debug('sorted versions: %s', res)

    # FIXME: the cases where get_content_version expects master on empty ver list are skipped
    if not res:
        return

    assert res[-1] == ver_sort_data['latest']


def test_normalize_versions():
    vers = ['v1.0.0', '1.0.1', 'V1.2.3', '3.4.5']
    norm_vers, norm_to_orig = repository_version.normalize_versions(vers)

    log.debug('     vers: %s', vers)
    log.debug('norm_vers: %s', norm_vers)

    for index, item in enumerate(vers):
        log.debug(' v: %s', item)
        log.debug('nv: %s', norm_vers[index])

    for norm_ver in norm_vers:
        assert not norm_ver.startswith('v')
        assert not norm_ver.startswith('V')

    assert '1.0.0' in norm_vers
    assert '1.2.3' in norm_vers

    log.debug('ntoo_map: %s', norm_to_orig)
    assert norm_to_orig[0][1] == 'v1.0.0'
    assert norm_to_orig[1][1] == '1.0.1'
    assert norm_to_orig[2][1] == 'V1.2.3'
    assert norm_to_orig[3][1] == '3.4.5'


@pytest.mark.parametrize("versions,exp_valid,exp_invalid", [
    (['v1.0.0', '1.0.1', 'V1.2.3', '3.4.5', 'a', '12', '1.0', '0.0.0.0', '0.0.0', '0.0'],
     ['0.0.0', '1.0.1', '3.4.5'],
     ['v1.0.0', 'V1.2.3', 'a', '12', '1.0', '0.0.0.0', '0.0']),
    (content_versions_alphanumeric_post,
     [],
     content_versions_alphanumeric_post),
    (['v1.0.0', 'v3.0.0', 'v0.0.0'],
     [],
     content_versions_v),
])
def test_validate_versions(versions, exp_valid, exp_invalid):
    valid, invalid = repository_version.validate_versions(versions)

    log.debug('valid: %s', valid)
    log.debug('invalid: %s', invalid)

    assert set(exp_valid) == set(valid)
    assert set(exp_invalid) == set(exp_invalid)


def test_get_content_version(ver_data):
    req_spec_data = {'namespace': 'some_namespace',
                     'name': 'some_name',
                     'version': ver_data['ask']}
    req_spec = RequirementSpec.from_dict(req_spec_data)

    res = repository_version.get_repository_version({}, req_spec, ver_data['vlist'])
    log.debug('res: %s', res)
    assert res == ver_data['exp']


latest_test_data = [
    {'vid': '1_0_0_latest', 'exp': '1.0.0'},
    {'vid': 'boring', 'exp': '3.0.0'},
    {'vid': 'no_v', 'exp': '3.0.0'},
]


@pytest.fixture(scope='module',
                params=latest_test_data,
                ids=['vlist=%s,exp=%s' % (x['vid'], x['exp']) for x in latest_test_data])
def latest_ver_data(request):
    tdata = request.param.copy()
    tdata['vlist'] = content_versions_map[tdata['vid']]
    log.debug('tdata: %s', tdata)
    yield tdata


def test_get_content_version_latest(latest_ver_data):
    log.debug('latest_ver_data: %s', latest_ver_data)
    req_spec_data = {'namespace': 'some_namespace',
                     'name': 'some_name',
                     'version_spec': '*'}
    req_spec = RequirementSpec.from_dict(req_spec_data)

    res = repository_version.get_repository_version({}, req_spec, latest_ver_data['vlist'])
    log.debug('res: %s', res)
    assert res == latest_ver_data['exp']


def test_get_latest_version():
    vers = ['1.0.0', '2.0.0', '2.0.1', '2.2.0', '3.1.4']
    res = repository_version.get_latest_version(vers,
                                                content_data={})

    assert res == '3.1.4'


def test_get_latest_version_invalid_semver():
    vers = ['1.0.0', '2.0.0', '2.0.1', '2.2.0', '3.1.4', '4.2']
    try:
        repository_version.get_latest_version(vers,
                                              content_data={})
    except exceptions.GalaxyClientError as e:
        assert 'Unable to compare' in '%s' % e
        assert "Invalid version string: '4.2'" in '%s' % e
        return

    assert False, 'Expected a GalaxyClientError about invalid versions here, but that did not happen.'


def test_get_latest_in_content_versions_1_0_0_v_and_no_v():
    req_spec_data = {'namespace': 'some_namespace',
                     'name': 'some_name',
                     'version_spec': '*'}
    req_spec = RequirementSpec.from_dict(req_spec_data)

    with pytest.raises(exceptions.GalaxyClientError, match=""):
        repository_version.get_repository_version({}, req_spec,
                                                  content_versions_1_0_v_and_no_v)



########################################################################
#
# (C) 2015, Brian Coca <bcoca@ansible.com>
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
########################################################################
''' This manages remote shared Ansible objects, mainly roles'''

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import logging

log = logging.getLogger(__name__)


class GalaxyContext(object):
    ''' Keeps global galaxy info '''

    def __init__(self, content_path=None, server=None):
        self.server = server or {'url': None,
                                 'ignore_certs': False}
        self.content_path = content_path

    def __repr__(self):
        return 'GalaxyContext(content_path=%s, server=%s)' % \
            (self.content_path, self.server)



import collections
import logging
import os

import pprint

from ansible_galaxy import collection_members

log = logging.getLogger(__name__)
pf = pprint.pformat

hello_path = os.path.join(os.path.dirname(__file__), 'collection_examples', 'hello')


def test_file_walker_cwd():
    cwd = os.getcwd()
    git_dir = os.path.join(cwd, '.git/')

    file_walker = collection_members.FileWalker(cwd, exclude_patterns=collection_members.DEFAULT_IGNORE_PATTERNS+['*.json'])
    file_names = file_walker.walk()
    for file_name in file_names:
        assert not file_name.startswith(git_dir), '%s found in %s but should be excluded by default' % (git_dir, file_name)
        assert '__pycache__' not in file_name, '__pycache__ found in path %s, but __pycache__ dirs should be ignored by default' % (file_name)


def test_file_walker_rel_path():

    file_walker = collection_members.FileWalker(hello_path,
                                                exclude_patterns=collection_members.DEFAULT_IGNORE_PATTERNS+['*.json'])
    rel_paths = file_walker.relative_walk()

    paths = []
    for rel_path in rel_paths:

        assert rel_path != 'some_json_file.json', \
            'Found some_json_file.json in relative_walk() results and it should be exclude'

        assert not rel_path.startswith(hello_path), 'rel_path (%s) starts with %s' % (rel_path, hello_path)

        assert not rel_path.startswith('releases'), 'releases subdir should be ignore but was found in %s' % rel_path
        paths.append(rel_path)

    log.debug(pf(sorted(paths)))


def test_collection_members_init():
    # gather files from mazer src dir, not really a role
    collection_path = os.path.join(os.getcwd())
    file_walker = collection_members.FileWalker(collection_path)
    coll_members = collection_members.CollectionMembers(walker=file_walker)
    log.debug('coll_members: %s', coll_members)

    members = coll_members.run()

    # NOTE: no reason .run() couldn't return a iterable/generator this is not
    #       actually an (sub)instance of collections.Iterable, but it is to catch
    #       when I forget to yield
    assert isinstance(members, collections.Iterable)

    members_list = list(members)
    assert isinstance(members_list, list)


def test_collection_members_post_filter():

    file_walker = collection_members.FileWalker(hello_path, exclude_patterns=['*.json'])
    coll_members = collection_members.CollectionMembers(walker=file_walker)

    members = coll_members.run()

    # just to trigger the iteration
    members_list = list(members)
    assert isinstance(members_list, list)

    log.debug('members_list: %s', pf(members_list))

    some_json_path = os.path.join(hello_path, 'some_json_file.json')
    assert some_json_path not in members_list



import logging

import hashlib

log = logging.getLogger(__name__)


def sha256sum_from_fo(fo):
    block_size = 65536
    sha256 = hashlib.sha256()
    for block in iter(lambda: fo.read(block_size), b''):
        sha256.update(block)
    return sha256.hexdigest()


def sha256sum_from_path(filename):
    with open(filename, 'rb') as fo:
        return sha256sum_from_fo(fo)




import logging

from ansible_galaxy import repository
from ansible_galaxy import repository_archive


log = logging.getLogger(__name__)


class LocalFileFetch(object):
    fetch_method = 'local_file'

    def __init__(self, requirement_spec):
        self.requirement_spec = requirement_spec
        self.local_path = self.requirement_spec.src

    def find(self):
        vspec = self.requirement_spec.version_spec.specs[0].spec

        results = {'content': {'galaxy_namespace': self.requirement_spec.namespace,
                               'repo_name': self.requirement_spec.name,
                               # For a local file, we created the version_spec to match
                               # exactly the version from the file, so dig into the version_spec
                               # a bit to pull that out.
                               # TODO/FIXME: helper method/wrapper for making this less coupled
                               'version': str(vspec)},
                   }
        return results

    def fetch(self, find_results=None):
        find_results = find_results or {}

        repository_archive_path = self.local_path

        log.debug('repository_archive_path=%s (inplace)', repository_archive_path)

        repo_archive = self._load_repository_archive(repository_archive_path)

        repo = self._load_repository(repo_archive)

        results = {'archive_path': repository_archive_path,
                   'fetch_method': self.fetch_method}

        results['custom'] = {'local_path': self.local_path}
        results['content'] = find_results['content']
        results['content']['fetched_name'] = repo.repository_spec.name

        return results

    def _load_repository_archive(self, archive_path):
        repo_archive = repository_archive.load_archive(archive_path)
        log.debug('repo_archive: %s', repo_archive)

        return repo_archive

    def _load_repository(self, repository_archive):
        repo = repository.load_from_archive(repository_archive)
        log.debug('repo: %s', repo)
        return repo

    def cleanup(self):
        return None



import logging
import yaml

import yamlloader

log = logging.getLogger(__name__)


def load(config_file_stream):
    conf_data = yaml.load(config_file_stream,
                          Loader=yamlloader.ordereddict.CSafeLoader)
    return conf_data


def save(conf_data, config_file_stream):
    result = yaml.dump(conf_data,
                       config_file_stream,
                       Dumper=yamlloader.ordereddict.CSafeDumper,
                       default_flow_style=False)
    return result
